"""This module contains internal logging utilities."""

# Copyright (C) 2008 Martin Sandve Alnes
# Copyright (C) 2014 Jan Blechta
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

from six import string_types
import io, logging, os, platform, sys

# Logging wrappers
_log = logging.getLogger("instant")
_loghandler = logging.StreamHandler()
_log.addHandler(_loghandler)
#_log.setLevel(logging.WARNING)
_log.setLevel(logging.INFO)
#_log.setLevel(logging.DEBUG)

# Choose method for calling external programs. use subprocess by
# default, and os.system on Windows
_default_call_method = 'SUBPROCESS'
if 'Windows' in platform.system() or 'CYGWIN' in platform.system():
    _default_call_method = 'OS_SYSTEM'
_call_method = os.environ.get("INSTANT_SYSTEM_CALL_METHOD",
                              _default_call_method)
_log.debug('Using call method: %s'%_call_method)


def get_log_handler():
    return _loghandler


def get_logger():
    return _log


def set_log_handler(handler):
    global _loghandler
    _log.removeHandler(_loghandler)
    _loghandler = handler
    _log.addHandler(_loghandler)


def set_logging_level(level):
    import inspect
    frame = inspect.currentframe().f_back
    instant_warning("set_logging_level is deprecated but was called "\
                    "from %s, at line %d. Use set_log_level instead." % \
                    (inspect.getfile(frame), frame.f_lineno))
    set_log_level(level)


def set_log_level(level):
    if isinstance(level, string_types):
        level = level.upper()
        assert level in ("INFO", "WARNING", "ERROR", "DEBUG")
        level = getattr(logging, level)
    else:
        assert isinstance(level, int)
    _log.setLevel(level)


# Aliases for calling log consistently:


def instant_debug(*message):
    _log.debug(*message)


def instant_info(*message):
    _log.info(*message)


def instant_warning(*message):
    _log.warning(*message)


def instant_error(*message):
    _log.error(*message)
    text = message[0] % message[1:]
    raise RuntimeError(text)


def instant_assert(condition, *message):
    if not condition:
        _log.error(*message)
        text = message[0] % message[1:]
        raise AssertionError(text)


# Utility functions for file handling:


def write_file(filename, text, mode="w"):
    "Write text to a file and close it."
    try:
        if isinstance(text, bytes):
            text = text.decode("utf8")
        with io.open(filename, mode, encoding="utf8") as f:
            f.write(text)
            f.flush()
    except IOError as e:
        instant_error("Can't open '%s': %s" % (filename, e))


if _call_method == 'SUBPROCESS':

    # NOTE: subprocess in Python 2 is not OFED-fork-safe! Check subprocess.py,
    #       http://bugs.python.org/issue1336#msg146685
    #       OFED-fork-safety means that parent should not
    #       touch anything between fork() and exec(),
    #       which is not met in subprocess module. See
    #       https://www.open-mpi.org/faq/?category=openfabrics#ofa-fork
    #       http://www.openfabrics.org/downloads/OFED/release_notes/OFED_3.12_rc1_release_notes#3.03
    # However, subprocess32 backports the fix from Python 3 to 2.7.
    if os.name == "posix" and sys.version_info[0] < 3:
        try:
            import subprocess32 as subprocess
        except:
            import subprocess
    else:
        import subprocess

    def get_status_output(cmd, input=None, cwd=None, env=None):
        if isinstance(cmd, string_types):
            cmd = cmd.strip().split()
        instant_debug("Running: " + str(cmd))

        # NOTE: This is not OFED-fork-safe! Check subprocess.py,
        #       http://bugs.python.org/issue1336#msg146685
        #       OFED-fork-safety means that parent should not
        #       touch anything between fork() and exec(),
        #       which is not met in subprocess module. See
        #       https://www.open-mpi.org/faq/?category=openfabrics#ofa-fork
        #       http://www.openfabrics.org/downloads/OFED/release_notes/OFED_3.12_rc1_release_notes#3.03
        pipe = subprocess.Popen(cmd, shell=False, cwd=cwd, env=env, stdout=subprocess.PIPE,
                     stderr=subprocess.STDOUT)

        (output, errout) = pipe.communicate(input=input)
        assert not errout

        status = pipe.returncode
        output = output.decode('utf-8') if sys.version_info[0] > 2 else output

        return (status, output)

elif _call_method == 'OS_SYSTEM':
    import tempfile
    from .paths import get_default_error_dir

    def get_status_output(cmd, input=None, cwd=None, env=None):
        # We don't need function with such a generality.
        # We only need output and return code.
        if not isinstance(cmd, string_types) or input is not None or \
            cwd is not None or env is not None:
            raise NotImplementedError(
                'This implementation (%s) of get_status_output does'
                ' not accept \'input\', \'cwd\' and \'env\' kwargs.'
                %_call_method)

        f = tempfile.NamedTemporaryFile(dir=get_default_error_dir(),
                                        delete=True)

        # Execute cmd with redirection
        cmd += ' > ' + f.name + ' 2>&1'
        instant_debug("Running: " + str(cmd))
        # NOTE: Possibly OFED-fork-safe, tests needed!
        status = os.system(cmd)

        output = f.read()
        f.close()

        output = output.decode('utf-8') if sys.version_info[0] > 2 else output
        return (status, output)
else:
    instant_error('Incomprehensible environment variable'
                  ' INSTANT_SYSTEM_CALL_METHOD=%s'%_call_method)



#!/usr/bin/env python
"Find potential bugs in instant by static code analysis."

# PyChecker skips previously loaded modules
import os, sys, glob, shutil, re, logging, subprocess, hashlib
try:
    import numpy
except:
    pass

import pychecker.checker
import instant




from __future__ import print_function
import pytest
import numpy
import time
from instant import inline_with_numpy

c_code = """
double sum (int n1, double* array1){
  double tmp = 0.0;
  for (int i=0; i<n1; i++) {
      tmp += array1[i];
  }
  return tmp;
}
"""

def test_compare_numpy():

    sum_func = inline_with_numpy(c_code, arrays = [['n1', 'array1']], cache_dir="test_cache")

    a = numpy.arange(10000000); a = numpy.sin(a)

    t1 = time.time()
    sum1 = sum_func(a)
    t2 = time.time()
    print('With instant:', t2 - t1, 'seconds')

    t1 = time.time()
    sum2 =  numpy.sum(a)
    t2 = time.time()
    print('With numpy:   ', t2-t1, 'seconds')

    difference = abs(sum1 - sum2)
    assert difference < 1.0e-12



"""This module contains helper functions for configuration using pkg-config."""

# Copyright (C) 2008-2009 Kent-Andre Mardal
# Copyright (C) 2008 Martin Sandve Alnes
# Copyright (C) 2011-2013 Johan Hake
# Copyright (C) 2011 Joachim Berdal Haga
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD
# license.

from six import string_types
import os
from .output import get_status_output
import re

# Global cache variables
_swig_binary_cache = None
_swig_version_cache = None
_pkg_config_installed = None
_header_and_library_cache = {}


def check_and_set_swig_binary(binary="swig", path=""):
    """ Check if a particular swig binary is available"""
    global _swig_binary_cache
    if not isinstance(binary, string_types):
        raise TypeError("expected a 'str' as first argument")
    if not isinstance(path, string_types):
        raise TypeError("expected a 'str' as second argument")
    swig_binary = os.path.join(path, binary)
    if swig_binary == _swig_binary_cache:
        return True

    result, output = get_status_output("%s -version"%swig_binary)
    if result != 0:
        return False

    # Set binary cache
    _swig_binary_cache = swig_binary

    # Reset SWIG version cache
    pattern = "SWIG Version (.*)"
    r = re.search(pattern, output)
    _swig_version_cache = r.groups(0)[0]

    return True


def get_swig_binary():
    "Return any cached swig binary"
    return _swig_binary_cache if _swig_binary_cache else "swig"


def get_swig_version():
    """ Return the current swig version in a 'str'"""
    global _swig_version_cache
    if _swig_version_cache is None:
        # Check for swig installation
        result, output = get_status_output("%s -version"%get_swig_binary())
        if result != 0:
            raise OSError("SWIG is not installed on the system.")
        pattern = "SWIG Version (.*)"
        r = re.search(pattern, output)
        _swig_version_cache = r.groups(0)[0]
    return _swig_version_cache


def check_swig_version(version, same=False):
    """Check the swig version

    Returns True if the version of the installed swig is equal or
    greater than the version passed to the function.

    If same is True, the function returns True if and only if the two versions
    are the same.

    Usage:
    if instant.check_swig_version('1.3.36'):
        print "Swig version is greater than or equal to 1.3.36"
    else:
        print "Swig version is lower than 1.3.36"

    """
    assert isinstance(version, string_types), "Provide the first version number as a 'str'"
    assert len(version.split(".")) == 3, "Provide the version number as three numbers seperated by '.'"

    installed_version = list(map(int, get_swig_version().split('.')))
    handed_version    = list(map(int, version.split('.')))

    # If same is True then just check that all numbers are equal
    if same:
        return all(i == h for i, h in zip(installed_version, handed_version))

    swig_enough = True
    for i, v in enumerate([v for v in installed_version]):
        if handed_version[i] < v:
            break
        elif handed_version[i] == v:
            continue
        else:
            swig_enough = False
        break

    return swig_enough


def header_and_libs_from_pkgconfig(*packages, **kwargs):
    """This function returns list of include files, flags, libraries and
    library directories obtain from a pkgconfig file.

    The usage is:
      (includes, flags, libraries, libdirs) = \
             header_and_libs_from_pkgconfig(*list_of_packages)
    or:
        (includes, flags, libraries, libdirs, linkflags) = \
             header_and_libs_from_pkgconfig(*list_of_packages, \
             returnLinkFlags=True)

    """

    global _pkg_config_installed, _header_and_library_cache
    returnLinkFlags = kwargs.get("returnLinkFlags", False)
    if _pkg_config_installed is None:
        result, output = get_status_output("pkg-config --version ")
        _pkg_config_installed = (result == 0)

    if not _pkg_config_installed:
        raise OSError("The pkg-config package is not installed on the system.")

    env = os.environ.copy()
    try:
        assert env["PKG_CONFIG_ALLOW_SYSTEM_CFLAGS"] == "0"
    except:
        env["PKG_CONFIG_ALLOW_SYSTEM_CFLAGS"] = "1"

    includes = []
    flags = []
    libs = []
    libdirs = []
    linkflags = []
    for pack in packages:
        if not pack in _header_and_library_cache:
            result, output = get_status_output(\
                "pkg-config --exists %s " % pack, env=env)
            if result == 0:
                tmp = get_status_output(\
                    "pkg-config --cflags-only-I %s " % pack, env=env)[1].split()
                _includes = [i[2:] for i in tmp]

                _flags = get_status_output(\
                    "pkg-config --cflags-only-other %s " % pack, env=env)[1].split()

                tmp = get_status_output(\
                    "pkg-config --libs-only-l  %s " % pack, env=env)[1].split()
                _libs = [i[2:] for i in tmp]

                tmp = get_status_output(\
                    "pkg-config --libs-only-L  %s " % pack, env=env)[1].split()
                _libdirs = [i[2:] for i in tmp]

                _linkflags = get_status_output(\
                    "pkg-config --libs-only-other  %s " % pack, env=env)[1].split()

                _header_and_library_cache[pack] = (_includes, _flags, _libs, \
                                                   _libdirs, _linkflags)
            else:
                _header_and_library_cache[pack] = None

        result = _header_and_library_cache[pack]
        if not result:
            raise OSError("The pkg-config file %s does not exist" % pack)

        _includes, _flags, _libs, _libdirs, _linkflags = result
        includes.extend(_includes)
        flags.extend(_flags)
        libs.extend(_libs)
        libdirs.extend(_libdirs)
        linkflags.extend(_linkflags)

    if returnLinkFlags:
        return (includes, flags, libs, libdirs, linkflags)

    return (includes, flags, libs, libdirs)



from __future__ import print_function
import pytest
import instant
from instant import build_module

c_code = """
double sum(double a, double b){
  return a+b;
}
"""

def test_build_module():
    build_module(code=c_code, modulename='test3_ext',
                 cppargs=['-pg', '-O3', '-g'], lddargs=['-pg'])

    from test3_ext import sum
    a = 3.7
    b = 4.8
    c = sum(a, b)
    assert c == 8.5



"""This module contains helper functions for working with temp and cache directories."""

# Copyright (C) 2008 Martin Sandve Alnes
# Copyright (C) 2012 Florian Rathgeber
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

# Utilities for directory handling:

from six import string_types
import os
import sys
import errno
import shutil
import tempfile
import time
from .signatures import compute_checksum
from .output import instant_debug, instant_assert

_tmp_dir = None


def get_temp_dir():
    """Return a temporary directory for the duration of this process.

    Multiple calls in the same process returns the same directory.
    Remember to call delete_temp_dir() before exiting."""
    global _tmp_dir
    if _tmp_dir is None:
        datestring = "%d-%d-%d-%02d-%02d" % time.localtime()[:5]
        suffix = datestring + "_instant_" + compute_checksum(get_default_cache_dir())
        _tmp_dir = tempfile.mkdtemp(suffix)
        instant_debug("Created temp directory '%s'." % _tmp_dir)
    return _tmp_dir


def delete_temp_dir():
    """Delete the temporary directory created by get_temp_dir()."""
    global _tmp_dir
    if _tmp_dir and os.path.isdir(_tmp_dir):
        shutil.rmtree(_tmp_dir, ignore_errors=True)
    _tmp_dir = None


def get_instant_dir():
    "Return the default instant directory, creating it if necessary."
    # Place default cache dir in virtualenv or conda prefix
    # if one of them are active, or under user's home directory
    home = os.path.expanduser("~")
    venv = os.environ.get("VIRTUAL_ENV")
    cenv = os.environ.get("CONDA_PREFIX")
    if venv == sys.prefix:
        env = venv
    elif cenv == sys.prefix:
        env = cenv
    else:
        env = home

    instant_dir = os.path.join(env, ".cache", "instant")

    # If placed in home directory, add python version for safety,
    # since C extensions are not compatible across versions.
    # (for python 3, it's possible to use the stable C API,
    # however we don't know if the instant user has done that)
    if env == home:
        ver = "python%d.%d" % sys.version_info[:2]
        instant_dir = os.path.join(instant_dir, ver)

    makedirs(instant_dir)
    return instant_dir


def get_default_cache_dir():
    "Return the default cache directory."
    cache_dir = os.environ.get("INSTANT_CACHE_DIR")
    # Catches the cases where INSTANT_CACHE_DIR is not set or ''
    if not cache_dir:
        cache_dir = os.path.join(get_instant_dir(), "cache")
    makedirs(cache_dir)
    return cache_dir


def get_default_error_dir():
    "Return the default error directory."
    error_dir = os.environ.get("INSTANT_ERROR_DIR")
    # Catches the cases where INSTANT_ERROR_DIR is not set or ''
    if not error_dir:
        error_dir = os.path.join(get_instant_dir(), "error")
    makedirs(error_dir)
    return error_dir


def validate_cache_dir(cache_dir):
    if cache_dir is None:
        return get_default_cache_dir()
    instant_assert(isinstance(cache_dir, string_types), "Expecting cache_dir to be a string.")
    cache_dir = os.path.abspath(cache_dir)
    makedirs(cache_dir)
    return cache_dir


def makedirs(path):
    """
    Creates a directory (tree). If directory already excists it does nothing.
    """
    try:
        os.makedirs(path)
        instant_debug("In instant.makedirs: Creating directory %r" % path)
    except os.error as e:
        if e.errno != errno.EEXIST:
            raise


def _test():
    from .output import set_logging_level
    set_logging_level("DEBUG")
    print("Temp dir:", get_temp_dir())
    print("Instant dir:", get_instant_dir())
    print("Default cache dir:", get_default_cache_dir())
    print("Default error dir:", get_default_error_dir())
    delete_temp_dir()


if __name__ == "__main__":
    _test()



"""This module contains helper functions for working with checksums."""

# Copyright (C) 2008 Martin Sandve Alnes
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

from six import string_types
import io
import hashlib
from .output import instant_assert, instant_debug, instant_error

def compute_checksum(text="", filenames=[]):
    """
    Get the checksum value of filename
    modified based on Python24\Tools\Scripts\md5.py
    """
    instant_assert(isinstance(text, string_types), "Expecting string.")
    instant_assert(isinstance(filenames, (list, tuple)), "Expecting sequence.")
    
    m = hashlib.new('sha1')
    if text:
        m.update(text.encode('utf-8'))
    
    for filename in sorted(filenames): 
        instant_debug("Adding file '%s' to checksum." % filename)
        try:
            fp = io.open(filename, 'rb')
        except IOError as e:
            instant_error("Can't open file '%s': %s" % (filename, e))
        
        try:
            while True:
                data = fp.read()
                if not data:
                    break
                m.update(data)
        except IOError as e:
            instant_error("I/O error reading '%s': %s" % (filename, e))
        
        fp.close() 
    
    return m.hexdigest().lower()


def _test():
    signature = "(Test signature)"
    files = ["signatures.py", "__init__.py"]
    print()
    print("Signature:", repr(signature))
    print("Checksum:", compute_checksum(signature, []))
    print()
    print("Files:", files)
    print("Checksum:", compute_checksum("", files))
    print()

if __name__ == "__main__":
    _test()




"""File locking for the cache system, to avoid problems
when multiple processes work with the same module.
Only works on UNIX systems.

Two Python libraries can be used:

  flufl.lock : A nfs safe which can be downloaded from:

                 https://launchpad.net/flufl.lock

  fcntl      : A builtin Python module which only works on posix machines
               and it is does unfortunately not work on nfs

"""

# Copyright (C) 2009 Martin Sandve Alnes
# Copyright (C) 2011-2013 Johan Hake
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

__all__ = ["get_lock", "release_lock", "release_all_lock", "file_lock"]

import os.path
from .output import instant_error, instant_assert, instant_debug
from .paths import validate_cache_dir

try:
    import flufl.lock
    fcntl = None
except:
    flufl = None
    try:
        import fcntl
    except:
        fcntl = None

# Keeping an overview of locks currently held, to avoid deadlocks
# within a single process
_lock_names = {} # lock.fileno() -> lockname
_lock_files = {} # lockname -> lock
_lock_count = {} # lockname -> number of times this lock has been aquired and not yet released

if flufl:
    def get_lock(cache_dir, module_name):
        "Get a new file lock."

        from flufl.lock import Lock
        from datetime import timedelta

        lockname = module_name + ".lock"
        count = _lock_count.get(lockname, 0)

        instant_debug("Acquiring lock %s, count is %d." % (lockname, count))

        cache_dir = validate_cache_dir(cache_dir)
        lockname = os.path.join(cache_dir, lockname)
        lock = Lock(lockname)
        lock.lock()

        return lock

    def release_lock(lock):
        "Release a lock currently held by Instant."
        if lock.is_locked:
            hostname, pid, lockname = lock.details
            instant_debug("Releasing lock %s." % (lockname))
            lock.unlock()

    def release_all_locks():
        pass

elif fcntl:
    def get_lock(cache_dir, module_name):
        "Get a new file lock."
        global _lock_names, _lock_files, _lock_count

        lockname = module_name + ".lock"
        count = _lock_count.get(lockname, 0)
        import inspect
        frame = inspect.currentframe().f_back
        instant_debug("Acquiring lock %s, count is %d. Called from: %s line: %d" % \
                      (lockname, count, inspect.getfile(frame), frame.f_lineno))

        if count == 0:
            cache_dir = validate_cache_dir(cache_dir)
            lock = open(os.path.join(cache_dir, lockname), "w")
            fcntl.flock(lock.fileno(), fcntl.LOCK_EX)
            _lock_names[lock.fileno()] = lockname
            _lock_files[lockname] = lock
        else:
            lock = _lock_files[lockname]

        _lock_count[lockname] = count + 1
        return lock

    def release_lock(lock):
        "Release a lock currently held by Instant."
        global _lock_names, _lock_files, _lock_count

        lockname = _lock_names[lock.fileno()]
        count = _lock_count[lockname]

        import inspect
        frame = inspect.currentframe().f_back

        instant_debug("Releasing lock %s, count is %d. Called from: %s line: %d" % \
                      (lockname, count, inspect.getfile(frame), frame.f_lineno))

        instant_assert(count > 0, "Releasing lock that Instant is supposedly not holding.")
        instant_assert(lock is _lock_files[lockname], "Lock mismatch, might be something wrong in locking logic.")

        del _lock_files[lockname]
        del _lock_names[lock.fileno()]
        _lock_count[lockname] = count - 1

        fcntl.flock(lock.fileno(), fcntl.LOCK_UN)
        lock.close()

    def release_all_locks():
        "Release all locks currently held by Instant."
        locks = list(_lock_files.values())
        for lock in locks:
            release_lock(lock)
        instant_assert(all(_lock_count[lockname] == 0 for lockname in _lock_count), "Lock counts not zero after releasing all locks.")

else:
    # Windows systems have no fcntl, implement these otherwise if
    # locking is needed on windows
    def get_lock(cache_dir, module_name):
        return None

    def release_lock(lock):
        pass

    def release_all_locks():
        pass


class file_lock(object):
    """
    File lock using with statement
    """
    def __init__(self, cache_dir, module_name):
        self.cache_dir = cache_dir
        self.module_name = module_name

    def __enter__(self):
        self.lock = get_lock(self.cache_dir, self.module_name)
        return self.lock

    def __exit__(self, type, value, tb):
        release_lock(self.lock)



from __future__ import print_function
import numpy
import time
from instant import inline_module_with_numpy

c_code = """
double sum (int n1, double* array1){
  double tmp = 0.0;
  for (int i=0; i<n1; i++) {
      tmp += array1[i];
  }
  return tmp;
}
double max(int n1, double* array1){
  double tmp = 0.0;
  for (int i=0; i<n1; i++) {
      if (array1[i] > tmp) {
          tmp = array1[i];
      }
  }
  return tmp;
}

"""


c_module = inline_module_with_numpy(c_code, arrays = [['n1', 'array1']], cache_dir="test_cache")


a = numpy.arange(10000000); a = numpy.sin(a)

sum_func = c_module.sum
max_func = c_module.max


sum = sum_func(a)
max = max_func(a)

print("sum ", sum)
print("max ", max)



from __future__ import print_function
import pytest
from instant import build_module
import numpy as N
import sys
import time


c_code = """
/* add function for matrices with all safety checks removed ..*/
void add(int x1, int y1, double* array1,
         int x2, int y2, double* array2,
         int x3, int y3, double* array3){

  for (int i=0; i<x1; i++) {
    for (int j=0; j<y1; j++) {
      *array3 = *array1 + *array2;
      array3++;
      array2++;
      array1++;
    }
  }
}
"""

def test_build_module():

    # Guess arrayobject is either in sys.prefix or /usr/local

    test6_ext = build_module(code=c_code, system_headers=["numpy/arrayobject.h"], cppargs=['-g'],
                             include_dirs=[N.get_include()],
                             init_code='import_array();', modulename='test6_ext',
                             arrays = [['x1', 'y1', 'array1'],
                                       ['x2', 'y2', 'array2'],
                                       ['x3', 'y3', 'array3']])

    from test6_ext import add
    a = N.arange(4000000); a = N.sin(a); a.shape=(2000, 2000)
    b = N.arange(4000000); b = N.cos(b); b.shape=(2000, 2000)
    c = N.arange(4000000); c = N.cos(c); c.shape=(2000, 2000)
    d = N.arange(4000000); d = N.cos(d); d.shape=(2000, 2000)

    t1 = time.time()
    add(a, b, c)
    t2 = time.time()

    t3 = time.time()
    N.add(a, b, d)
    t4 = time.time()

    e = abs(d-c)
    e.shape=(4000000,)

    max_difference = max(e)
    assert abs(max_difference) < 1.0e-12



"""This module contains the main part of Instant, the build_module function."""

# Copyright (C) 2008-2013 Kent-Andre Mardal
# Copyright (C) 2008-2013 Martin Sandve Alnes
# Copyright (C) 2009-2014 Johan Hake
# Copyright (C) 2009 Ilmar Wilbers
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

import six
from six import string_types

import io, os, sys, shutil, glob, errno
from itertools import chain

# TODO: Import only the official interface
from .output import *
from .paths import *
from .signatures import *
from .cache import *
from .codegeneration import *
from .locking import file_lock


def assert_is_str(x):
    instant_assert(isinstance(x, string_types), "In instant.build_module: Expecting string.")


def assert_is_bool(x):
    instant_assert(isinstance(x, bool), "In instant.build_module: Expecting bool.")


def assert_is_str_list(x):
    instant_assert(isinstance(x, (list, tuple)), "In instant.build_module: Expecting sequence.")
    instant_assert(all(isinstance(i, string_types) for i in x), "In instant.build_module: Expecting sequence of strings.")


def strip_strings(x):
    assert_is_str_list(x)
    return [s.strip() for s in x]


def arg_strings(x):
    if isinstance(x, string_types):
        x = x.split()
    return strip_strings(x)


def makedirs(path):
    """
    Creates a directory (tree). If directory already excists it does nothing.
    """
    try:
        os.makedirs(path)
    except os.error as e:
        if e.errno != errno.EEXIST:
            raise


def copy_files(source, dest, files):
    """Copy a list of files from a source directory to a destination
    directory.  This may seem a bit complicated, but a lot of this
    code is error checking.

    """
    if os.path.exists(dest):
        overwriting = set(files) & set(glob.glob(os.path.join(dest, "*")))
        if overwriting:
            instant_warning("In instant.copy_files: Path '%s' already exists, "\
                "overwriting existing files: %r." % (dest, list(overwriting)))
    else:
        makedirs(dest)

    if source != dest:
        instant_debug("In instant.copy_files: Copying files %r from %r to %r"\
            % (files, source, dest))

        for f in files:
            a = os.path.join(source, f)
            b = os.path.join(dest, f)
            instant_assert(a != b, "In instant.copy_files: Seems like the "\
                "input files are absolute paths, should be relative to "\
                "source. (%r, %r)" % (a, b))
            instant_assert(os.path.isfile(a), "In instant.copy_files: "\
                "Missing source file '%s'." % a)
            if os.path.isfile(b):
                os.remove(b)
            shutil.copyfile(a, b)


def recompile(modulename, module_path, new_compilation_checksum,
              build_system="distutils"):
    """Recompile module if the new checksum is different from
    the one in the checksum file in the module directory."""

    assert(build_system in ["distutils", "cmake"])
    # Check if the old checksum matches the new one
    compilation_checksum_filename = "%s.checksum" % modulename
    if os.path.exists(compilation_checksum_filename):
        checksum_file = io.open(compilation_checksum_filename, encoding="utf8")
        old_compilation_checksum = checksum_file.readline()
        checksum_file.close()
        if old_compilation_checksum == new_compilation_checksum:
            return

    # Create log file for logging of compilation errors
    compile_log_filename = os.path.join(module_path, "compile.log")
    compile_log_filename_dest = os.path.join(get_default_error_dir(), \
                                             modulename, "compile.log")

    ret = 1
    try:
        compile_log_contents = None
        instant_info("--- Instant: compiling ---")

        # TODO: The three blocks below can be made a function and
        # three calls
        if build_system == "distutils":
            # Build extension module with distutils
            python_interp = sys.executable
            cmd = python_interp + " setup.py build_ext install --install-platlib=."
            instant_debug("cmd = %s" % cmd)
            ret, output = get_status_output(cmd)
            write_file(compile_log_filename, output)
            if ret != 0:
                compile_log_contents = output
                if os.path.exists(compilation_checksum_filename):
                    os.remove(compilation_checksum_filename)
                msg = "In instant.recompile: The module did not compile with command '%s', see '%s'"
                instant_error(msg % (cmd, compile_log_filename_dest))
        else:
            # Build makefile for extension module with cmake
            cmd = "cmake -DDEBUG=TRUE .";
            #cmd = "cmake .";
            instant_debug("cmd = %s" % cmd)
            ret, output = get_status_output(cmd)
            write_file(compile_log_filename, output)
            if ret != 0:
                compile_log_contents = output
                if os.path.exists(compilation_checksum_filename):
                    os.remove(compilation_checksum_filename)
                msg = "In instant.recompile: The module did not compile with command '%s', see '%s'"
                instant_error(msg % (cmd, compile_log_filename_dest))

            # Build extension module with cmake generated makefile
            cmd = "make VERBOSE=1"
            instant_debug("cmd = %s" % cmd)
            ret, output = get_status_output(cmd)
            write_file(compile_log_filename, output, mode="a")
            if ret != 0:
                compile_log_contents = output
                if os.path.exists(compilation_checksum_filename):
                    os.remove(compilation_checksum_filename)
                msg = "In instant.recompile: The module did not compile with command '%s', see '%s'"
                instant_error(msg % (cmd, compile_log_filename_dest))

    finally:
        if ret != 0:
            if "INSTANT_DISPLAY_COMPILE_LOG" in list(os.environ.keys()):
                instant_warning("")
                instant_warning("Content of instant compile.log")
                instant_warning("==============================")
                instant_warning(compile_log_contents)
                instant_warning("")

            # Copy module to error dir
            module_path = copy_to_cache(module_path, get_default_error_dir(),
                                        modulename,
                                        check_for_existing_path=False)

    # Compilation succeeded, write new_compilation_checksum to
    # checksum_file
    write_file(compilation_checksum_filename, new_compilation_checksum)


def copy_to_cache(module_path, cache_dir, modulename,
                  check_for_existing_path=True):
    "Copy module directory to cache."
    # Get lock, check if the module exists, _otherwise_ copy the
    # finished compiled module from /tmp/foo to the cache directory,
    # and then release lock
    with file_lock(cache_dir, modulename) as lock:

        # Validate the path
        cache_module_path = os.path.join(cache_dir, modulename)
        if check_for_existing_path and os.path.exists(os.path.join(\
            cache_module_path, "finished_copying")):
            # This indicates a race condition has happened (and is
            # being avoided!).
            instant_warning("In instant.build_module: Path '%s' already exists,"\
                " but module wasn't found in cache previously. Not overwriting,"\
                " assuming this module is valid." % cache_module_path)

            return cache_module_path

            # Not deleting anymore, relying on locking system
            #shutil.rmtree(cache_module_path, ignore_errors=True)

        # Error checks
        instant_assert(os.path.isdir(module_path), "In instant.build_module:"\
                       " Cannot copy non-existing directory %r!" % module_path)
        if check_for_existing_path and os.path.isdir(cache_module_path):
            instant_error("In instant.build_module: Cache directory %r shouldn't"\
                          " exist at this point!" % cache_module_path)
        instant_debug("In instant.build_module: Copying built module from %r"\
            " to cache at %r" % (module_path, cache_module_path))

        # Do the copying and mark that we are finished by creating an
        # empty file finished_copying
        try:
            shutil.copytree(module_path, cache_module_path)
            with io.open(os.path.join(cache_module_path, "finished_copying"),
                             "w", encoding="utf8") as dummy:
                pass            
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise
        finally:
            delete_temp_dir()

    return cache_module_path


def build_module(modulename=None, source_directory=".",
                 code="", init_code="",
                 additional_definitions="", additional_declarations="",
                 sources=[], wrap_headers=[],
                 local_headers=[], system_headers=[],
                 include_dirs=['.'], library_dirs=[], libraries=[],
                 swigargs=['-c++', '-fcompact', '-O', '-I.', '-small'],
                 swig_include_dirs = [],
                 cppargs=['-O2'], lddargs=[],
                 object_files=[], arrays=[],
                 generate_interface=True, generate_setup=True,
                 cmake_packages=[],
                 signature=None, cache_dir=None):
    """Generate and compile a module from C/C++ code using SWIG.

    Arguments:
    ==========
    The keyword arguments are as follows:
      - B{modulename}:
        - The name you want for the module.
          If specified, the module will not be cached.
          If missing, a name will be constructed based on
          a checksum of the other arguments, and the module
          will be placed in the global cache. String.
      - B{source_directory}:
        - The directory where user supplied files reside. The files
        given in B{sources}, B{wrap_headers}, and B{local_headers}
        are expected to exist in this directory. String.
      - B{code}:
        - A string containing C or C++ code to be compiled and wrapped. String.
      - B{init_code}:
        - Code that should be executed when the Instant module is
        imported. This code is inserted in the SWIG interface file, and is
        used for instance for calling C{import_array()} used for the
        initialization of NumPy arrays. String.
      - B{additional_definitions}:
        - Additional definitions (typically needed for inheritance)
        for interface file. These definitions should be given as triple-quoted
        strings in the case they span multiple lines, and are placed both in the
        initial block for C/C++ code (C{%{,%}}-block), and the main section
        of the interface file. String.
      - B{additional_declarations}:
        - Additional declarations (typically needed for inheritance)
        for interface file. These declarations should be given as triple-quoted
        strings in the case they span multiple lines, and are plaves in the main
        section of the interface file. String.
      - B{sources}:
        - Source files to compile and link with the module. These
        files are compiled togehter with the SWIG-generated wrapper file into
        the final library file. Should reside in directory specified in
        B{source_directory}. List of strings.
      - B{wrap_headers}:
        - Local header files that should be wrapped by SWIG. The
        files specified will be included both in the initial block for C/C++ code
        (with a C directive) and in the main section of the interface file (with
        a SWIG directive). Should reside in directory specified in
        B{source_directory}. List of strings.
      - B{local_headers}:
        - Local header files required to compile the wrapped
        code. The files specified will be included in the initial block for
        C/C++ code (with a C directive). Should reside in directory specified in
        B{source_directory}. List of strings.
      - B{system_headers}:
        - System header files required to compile the wrapped
        code. The files specified will be included in the initial block for C/C++
        code (with a C directive). List of strings.
      - B{include_dirs}:
        - Directories to search for header files for building the
        extension module. Needs to be absolute path names. List of strings.
      - B{library_dirs}:
        - Directories to search for libraries (C{-l}) for building
        the extension module. Needs to be absolute paths. List of strings.
      - B{libraries}:
        -  Libraries needed by the Instant module. The libraries will
        be linked in from the shared object file. The initial C{-l} is added
        automatically. List of strings.
      - B{swigargs}:
        - List of arguments to swig, e.g. C{["-lpointers.i"]}
          to include the SWIG pointers.i library.
      - B{swig_include_dirs}:
        - A list of directories to include in the 'swig' command.
      - B{cppargs}:
        - List of arguments to the compiler, e.g. C{["-Wall", "-fopenmp"]}.
      - B{lddargs}:
        - List of arguments to the linker, e.g. C{["-E", "-U"]}.
      - B{object_files}:
        - If you want to compile the files yourself. TODO: Not yet supported.
      - B{arrays}:
        - A nested list describing the C arrays to be made from NumPy arrays.
        The SWIG interface for fil NumPy is used. For 1D arrays, the inner
        list should contain strings with the variable names for the length of
        the arrays and the array itself. 2D matrices should contain the names
        of the dimensions in the two directions as well as the name of the
        array, and 3D tensors should contain the names of the dimensions in
        the three directions in addition to the name of the array.
        If the NumPy array har more than four dimensions, the inner list should
        contain strings with variable names for the number of dimensions,
        the length in each dimension as a pointer, and the array itself, respectively.
      - B{generate_interface}:
        - A bool to indicate if you want to generate the interface files.
      - B{generate_setup}:
        - A bool to indicate if you want to generate the setup.py file.
      - B{cmake_packages}:
        - A list with CMake configured packages which are used to configure
        and build the extension module. If used it will override the default
        behaviour of using distutils.
      - B{signature}:
        - A signature string to identify the form instead of the source code.
      - B{cache_dir}:
        - A directory to look for cached modules and place new ones.
          If missing, a default directory is used. Note that the module
          will not be cached if B{modulename} is specified.
          The cache directory should not be used for anything else.
    """

    # Store original directory to be able to restore later
    original_path = os.getcwd()

    # --- Validate arguments

    if sys.version_info[0] > 2:
        swigargs = swigargs + ['-py3']

    instant_assert(modulename is None or isinstance(modulename, string_types),
        "In instant.build_module: Expecting modulename to be string or None.")
    assert_is_str(source_directory)
    source_directory = os.path.abspath(source_directory)
    assert_is_str(code)
    assert_is_str(init_code)
    assert_is_str(additional_definitions)
    assert_is_str(additional_declarations)
    sources           = strip_strings(sources)
    wrap_headers      = strip_strings(wrap_headers)
    local_headers     = strip_strings(local_headers)
    system_headers    = strip_strings(system_headers)
    include_dirs      = strip_strings(include_dirs)
    library_dirs      = strip_strings(library_dirs)
    libraries         = strip_strings(libraries)
    swigargs          = arg_strings(swigargs)
    swig_include_dirs = strip_strings(swig_include_dirs)
    cppargs           = arg_strings(cppargs)
    lddargs           = arg_strings(lddargs)
    object_files      = strip_strings(object_files)
    arrays            = [strip_strings(a) for a in arrays]
    assert_is_bool(generate_interface)
    assert_is_bool(generate_setup)
    cmake_packages   = strip_strings(cmake_packages)

    instant_assert(signature is None \
                   or isinstance(signature, string_types) \
                   or hasattr(signature, "signature"), "In instant.build_module: Expecting modulename to be string or None.")

    instant_assert(not (signature is not None and modulename is not None), "In instant.build_module: Can't have both modulename and signature.")

    # --- Replace arguments with defaults if necessary

    cache_dir = validate_cache_dir(cache_dir)

    # Split sources by file-suffix (.c or .cpp)
    csrcs = [f for f in sources if f.endswith('.c') or f.endswith('.C')]
    cppsrcs = [f for f in sources if f.endswith('.cpp') or f.endswith('.cxx')]
    instant_assert(len(csrcs) + len(cppsrcs) == len(sources), "In instant.build_module: Source files must have '.c' or '.cpp' suffix")

    # --- Debugging code
    instant_debug('In instant.build_module:')
    instant_debug('::: Begin Arguments :::')
    instant_debug('    modulename: %r' % modulename)
    instant_debug('    source_directory: %r' % source_directory)
    instant_debug('    code: %r' % code)
    instant_debug('    init_code: %r' % init_code)
    instant_debug('    additional_definitions: %r' % additional_definitions)
    instant_debug('    additional_declarations: %r' % additional_declarations)
    instant_debug('    sources: %r' % sources)
    instant_debug('    csrcs: %r' % csrcs)
    instant_debug('    cppsrcs: %r' % cppsrcs)
    instant_debug('    wrap_headers: %r' % wrap_headers)
    instant_debug('    local_headers: %r' % local_headers)
    instant_debug('    system_headers: %r' % system_headers)
    instant_debug('    include_dirs: %r' % include_dirs)
    instant_debug('    library_dirs: %r' % library_dirs)
    instant_debug('    libraries: %r' % libraries)
    instant_debug('    swigargs: %r' % swigargs)
    instant_debug('    swig_include_dirs: %r' % swig_include_dirs)
    instant_debug('    cppargs: %r' % cppargs)
    instant_debug('    lddargs: %r' % lddargs)
    instant_debug('    object_files: %r' % object_files)
    instant_debug('    arrays: %r' % arrays)
    instant_debug('    generate_interface: %r' % generate_interface)
    instant_debug('    generate_setup: %r' % generate_setup)
    instant_debug('    cmake_packages: %r' % cmake_packages)
    instant_debug('    signature: %r' % signature)
    instant_debug('    cache_dir: %r' % cache_dir)
    instant_debug('::: End Arguments :::')

    # --- Setup module directory, making it and copying
    #     files to it if necessary, and compute a modulename
    #     if it isn't specified explicitly

    if modulename is None:
        # Compute a signature if we have none passed by the user:
        if signature is None:
            # Collect arguments used for checksum creation,
            # including everything that affects the interface
            # file generation and module compilation.
            checksum_args = ( \
                # We don't care about the modulename, that's what
                # we're trying to construct!
                #modulename,
                # We don't care where the user code resides:
                #source_directory,
                code, init_code,
                additional_definitions,
                additional_declarations,
                # Skipping filenames, since we use the file contents:
                #sources, wrap_headers,
                #local_headers,
                system_headers,
                include_dirs, library_dirs, libraries,
                swig_include_dirs, swigargs, cppargs, lddargs,
                object_files, arrays,
                generate_interface, generate_setup, cmake_packages,
                # The signature isn't defined, and the cache_dir
                # doesn't affect the module:
                #signature, cache_dir)
                sys.version
            )
            allfiles = sources + wrap_headers + local_headers
            allfiles = [os.path.join(source_directory, f) for f in allfiles]
            text = "\n".join((str(a) for a in checksum_args))
            signature = modulename_from_checksum(compute_checksum(text, allfiles))
            modulename = signature
            moduleids = [signature]
        else:
            module, moduleids = check_memory_cache(signature)
            if module: return module
            modulename = moduleids[-1]

        # Look for module in disk cache
        module = check_disk_cache(modulename, cache_dir, moduleids)
        if module: return module

        # Make a temporary module path for compilation
        module_path = os.path.join(get_temp_dir(), modulename)
        instant_assert(not os.path.exists(module_path), "In instant.build_module: Not expecting module_path to exist: '%s'"\
            % module_path)
        makedirs(module_path)
        use_cache = True
    else:
        use_cache = False
        moduleids = []
        module_path = os.path.join(original_path, modulename)
        makedirs(module_path)

        ## Look for module in memory cache
        #module, moduleids = check_memory_cache(modulename)
        #if module: return module
        #instant_assert(modulename == moduleids[-1] and len(moduleids) == 1, "Logic breach.")
        ## Look for module in local directory
        #module = check_disk_cache(modulename, original_path, moduleids)
        #if module: return module

    # Wrapping rest of code in try-block to
    # clean up at the end if something fails.
    try:
        # --- Copy user-supplied files to module path

        module_path = os.path.abspath(module_path)
        files_to_copy = sources + wrap_headers + local_headers + object_files
        copy_files(source_directory, module_path, files_to_copy)
        # At this point, all user input files should reside in module_path

        # --- Generate additional files in module directory
        os.chdir(module_path)

        # Generate __init__.py which imports compiled module contents
        write_file("__init__.py", "from __future__ import absolute_import\nfrom .%s import *" \
                    % modulename)

        # Generate SWIG interface if wanted
        ifile_name = "%s.i" % modulename
        if generate_interface:
            write_interfacefile(ifile_name, modulename, code, init_code,
                                additional_definitions,
                                additional_declarations, system_headers,
                                local_headers, wrap_headers, arrays)

        # Generate setup.py if wanted
        if generate_setup and not cmake_packages:
            setup_name = "setup.py"
            write_setup(setup_name, modulename, csrcs, cppsrcs, local_headers,
                        include_dirs, library_dirs, libraries,
                        swig_include_dirs, swigargs, cppargs, lddargs)
            build_system = "distutils"

        else:
            write_cmakefile(modulename, cmake_packages, csrcs, cppsrcs,
                            local_headers, include_dirs, library_dirs,
                            libraries, swig_include_dirs, swigargs, cppargs,
                            lddargs)
            build_system = "cmake"

        # --- Build module

        # At this point we have all the files, and can make the total
        # checksum from all file contents. This is used to decide
        # whether the module needs recompilation or not.

        # Compute new_compilation_checksum
        # Collect arguments used for checksum creation,
        # including everything that affects the module compilation.
        # Since the interface file is included in allfiles,
        # we don't need stuff that modifies it here.
        checksum_args = ( \
                         # We don't care about the modulename, that's what
                         # we're trying to construct!
                         #modulename,
                         # We don't care where the user code resides:
                         #source_directory,
                         #code, init_code,
                         #additional_definitions, additional_declarations,
                         # Skipping filenames, since we use the file contents:
                         #sources, wrap_headers,
                         #local_headers,
                         system_headers,
                         include_dirs, library_dirs, libraries,
                         swigargs, swig_include_dirs, cppargs, lddargs,
                         object_files, #arrays,
                         #generate_interface, generate_setup,
                         # The signature isn't defined, and the
                         # cache_dir doesn't affect the module:
                         #signature, cache_dir)
                         )
        text = "\n".join((str(a) for a in checksum_args))
        allfiles = sources + wrap_headers + local_headers + [ifile_name]
        new_compilation_checksum = compute_checksum(text, allfiles)

        # Recompile if necessary
        recompile(modulename, module_path, new_compilation_checksum,
                  build_system)

        # --- Load, cache, and return module

        # Copy compiled module to cache
        if use_cache:
            module_path = copy_to_cache(module_path, cache_dir, modulename)

        # Import module and place in memory cache
        module = import_and_cache_module(module_path, modulename, moduleids)

        if not module:
            instant_error("Failed to import newly compiled module!")

        instant_debug("In instant.build_module: Returning %s from build_module."\
            % module)

        return module
        # The end!

    finally:
        # Always get back to original directory.
        os.chdir(original_path)

    instant_error("In instant.build_module: Should never reach this point!")
    # end build_module


def build_module_vtk(c_code, cache_dir=None):
    original_path = os.getcwd()
    cache_dir = validate_cache_dir(cache_dir)
    signature = modulename_from_checksum(compute_checksum(c_code))
    modulename = signature
    moduleids = [signature]
    module_path = os.path.join(get_temp_dir(), modulename)

    makedirs(module_path)
    os.chdir(module_path)

    write_cmakefile(modulename)
    s = generate_interface_file_vtk(signature, c_code)
    write_vtk_interface_file(signature, c_code)

    ret, output = get_status_output("cmake -DDEBUG=TRUE . > cmake.log ")
    ret, output = get_status_output("make > compile.log ")

    module_path = copy_to_cache(module_path, cache_dir, modulename)

    os.chdir(original_path)

    module = import_and_cache_module(module_path, modulename, moduleids)

    return module


def build_module_vmtk(c_code, cache_dir=None):
    original_path = os.getcwd()
    cache_dir = validate_cache_dir(cache_dir)
    signature = modulename_from_checksum(compute_checksum(c_code))
    modulename = signature
    moduleids = [signature]
    module_path = os.path.join(get_temp_dir(), modulename)

    makedirs(module_path)
    os.chdir(module_path)

    write_vmtk_cmakefile(modulename)
    s = generate_interface_file_vtk(signature, c_code)
    write_vtk_interface_file(signature, c_code)

    ret, output = get_status_output("cmake -DDEBUG=TRUE . > cmake.log ")
    ret, output = get_status_output("make > compile.log ")

    module_path = copy_to_cache(module_path, cache_dir, modulename)

    os.chdir(original_path)

    module = import_and_cache_module(module_path, modulename, moduleids)

    return module



from __future__ import print_function
from instant import inline_with_numpy
from numpy import *
import time
import os

c_code = r"""
void compute(int n, double* x, 
             int m, double*y) {   
    
    if ( n != m ) {
        printf("n and m should be equal");  
        return; 
    }
    #pragma omp parallel
    {
    int id;
    id = omp_get_thread_num(); 
    printf("Thread %d\n", id);

    #pragma omp for
    for (int i=0; i<m; i++) {
      y[i] =  sin(x[i]) +  cos(x[i]) + 3.4*x[i];  
    }
    }
}
"""

c_code_scalar  = r"""
void compute_scalar(int n, double* x, 
             int m, double*y) {   
    
    if ( n != m ) {
        printf("n and m should be equal");  
        return; 
    }
    for (int i=0; i<m; i++) {
      y[i] =  sin(x[i]);  
    }
}
"""




N = 8000000

compute_func = inline_with_numpy(c_code, arrays = [['n', 'x'], ['m', 'y']], cppargs = ['-fopenmp'], lddargs=['-lgomp'], system_headers=["omp.h"])  

x = arange(0, 1, 1.0/N) 
y = arange(0, 1, 1.0/N) 
t1 = time.time()
t3 = time.clock()
compute_func(x, y)
t2 = time.time()
t4 = time.clock()
print('With instant and OpenMP', t4-t3, 'seconds process time')
print('With instant and OpenMP', t2-t1, 'seconds wall time')

compute_func_scalar = inline_with_numpy(c_code_scalar, arrays = [['n', 'x'], ['m', 'y']])  

x = arange(0, 1, 1.0/N) 
y = arange(0, 1, 1.0/N) 
t5 = time.time()
t7 = time.clock()
compute_func_scalar(x, y)
t6 = time.time()
t8 = time.clock()
print('With instant ', t8-t7, 'seconds process time')
print('With instant ', t6-t5, 'seconds wall time')

print("")
print('Speed-up ', (t6-t5)/(t2-t1), 'seconds wall time')








from __future__ import print_function
import pytest
import numpy
import time
import instant
from instant import inline_with_numpy

def test_numpy_inline():

    c_code = """
double sum_of_some_func(int n1, double* array1){
  double tmp = 0.0;
  for (int i=0; i<n1; i++) {
      tmp += some_func(array1[i]);
  }
  return tmp;
}
    """

    some_func = inline_with_numpy(c_code, arrays=[['n1', 'array1']],
                                  local_headers=["./some_func.h"],
                                  libraries = ["m"])

    a = numpy.arange(10000000); a = numpy.sin(a)

    t1 = time.time()
    b = some_func(a)
    t2 = time.time()
    print('With instant:', t2-t1, 'seconds')

    t1 = time.time()
    c = sum(numpy.sin(a) + numpy.cos(a))
    t2 = time.time()
    print('With  numpy :', t2-t1, 'seconds')

    assert abs(c - b) < 1.0e-12



from __future__ import print_function
import pytest
from time import time
from instant import build_module, import_module

def test_repeated_build():
    #_t = None
    def tic():
        global _t
        _t = -time()

    def toc(msg=""):
        t = time() + _t
        print("t = %f  (%s)" % (t, msg))
        return t

    c_code = """
double sum(double a, double b)
{
  return a+b;
}
    """

    # Time a few builds
    tic()
    module = build_module(code=c_code)
    t1 = toc("first build")

    tic()
    module = build_module(code=c_code)
    t2 = toc("second build")

    tic()
    module = build_module(code=c_code)
    t3 = toc("third build")

    assert t1 > t2
    assert t1 > t3



"""This module contains helper functions for code generation."""

# Copyright (C) 2010, 2013 Kent-Andre Mardal
# Copyright (C) 2008 Martin Sandve Alnes
# Copyright (C) 2009 Ilmar Wilbers
# Copyright (C) 2013 Garth N. Wells
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

import sys
import re, os, io
from .output import instant_assert, instant_warning, instant_debug, write_file
from .config import get_swig_binary

def mapstrings(format, sequence):
    return "\n".join(format % i for i in sequence)

def reindent(code):
    '''Reindent a multiline string to allow easier to read syntax.

    Each line will be indented relative to the first non-empty line.
    Start the first line without text like shown in this example::

        code = reindent("""
            Foo
            Bar
                Blatti
            Ping
            """)

    makes all indentation relative to Foo.
    '''
    lines = code.split("\n")
    space = ""
    # Get initial spaces from first non-empty line:
    for l in lines:
        if l:
            r = re.search(r"^( [ ]*)", l)
            if r is not None:
                space = r.groups()[0]
            break
    if not space:
        return code
    n = len(space)
    instant_assert(space == " "*n, "Logic breach in reindent.")
    return "\n".join(re.sub(r"^%s" % space, "", l) for l in lines)

def write_interfacefile(filename, modulename, code, init_code,
                        additional_definitions, additional_declarations,
                        system_headers, local_headers, wrap_headers, arrays):
    """Generate a SWIG interface file. Intended for internal library use.

    The input arguments are as follows:
      - modulename (Name of the module)
      - code (Code to be wrapped)
      - init_code (Code to put in the init section of the interface file)
      - additional_definitions (Definitions to be placed in initial block with
        C code as well as in the main section of the SWIG interface file)
      - additional_declarations (Declarations to be placed in the main section
        of the SWIG interface file)
      - system_headers (A list of system headers with declarations needed by the wrapped code)
      - local_headers (A list of local headers with declarations needed by the wrapped code)
      - wrap_headers (A list of local headers that will be included in the code and wrapped by SWIG)
      - arrays (A nested list, the inner lists describing the different arrays)

    The result of this function is that a SWIG interface with
    the name modulename.i is written to the current directory.
    """
    instant_debug("Generating SWIG interface file '%s'." % filename)

    # create typemaps
    typemaps = ""
    valid_types = ['float', 'double', 'short', 'int', 'long', 'long long',
                   'unsigned short', 'unsigned int', 'unsigned long',
                   'unsigned long long']
    for a in arrays:
        if isinstance(a, tuple):
            a = list(a)
        DATA_TYPE = 'double'
        for vt in valid_types:
            if vt in a:
                DATA_TYPE = vt
                a.remove(vt)
        if 'in' in a:
            # input arrays
            a.remove('in')
            instant_assert(len(a) > 1 and len(a) < 5,
                           "Wrong number of elements in input array")
            if len(a) == 2:
                # 1-dimensional arrays, i.e. vectors
                typemaps += reindent("""
                %%apply (int DIM1, %(dtype)s* IN_ARRAY1) {(int %(n1)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'array' : a[1], 'dtype' : DATA_TYPE })
            elif len(a) == 3:
                # 2-dimensional arrays, i.e. matrices
                typemaps += reindent("""
                %%apply (int DIM1, int DIM2, %(dtype)s* IN_ARRAY2) {(int %(n1)s, int %(n2)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'n2' : a[1], 'array' : a[2], 'dtype' : DATA_TYPE })
            else:
                # 3-dimensional arrays, i.e. tensors
                typemaps += reindent("""
                %%apply (int DIM1, int DIM2, int DIM3, %(dtype)s* IN_ARRAY3) {(int %(n1)s, int %(n2)s, int %(n3)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'n2' : a[1], 'n3' : a[2], 'array' : a[3], 'dtype' : DATA_TYPE })
        elif 'out' in a:
            # output arrays
            a.remove('out')
            instant_assert(len(a) == 2, "Output array must be 1-dimensional")
            # 1-dimensional arrays, i.e. vectors
            typemaps += reindent("""
            %%apply (int DIM1, %(dtype)s* ARGOUT_ARRAY1) {(int %(n1)s, %(dtype)s* %(array)s)};
            """ % { 'n1' : a[0], 'array' : a[1], 'dtype' : DATA_TYPE })
        else:
            # in-place arrays
            instant_assert(len(a) > 1 and len(a) < 5,
                           "Wrong number of elements in output array")
            if 'multi' in a:
                # n-dimensional arrays, i.e. tensors > 3-dimensional
                a.remove('multi')
                typemaps += reindent("""
                %%typemap(in) (int %(n)s,int* %(ptv)s,%(dtype)s* %(array)s){
                  if (!PyArray_Check($input)) {
                    PyErr_SetString(PyExc_TypeError, "Not a NumPy array");
                    return NULL; ;
                  }
                  PyArrayObject* pyarray;
                  pyarray = (PyArrayObject*)$input;
                  $1 = int(pyarray->nd);
                  int* dims = new int[$1];
                  for (int d=0; d<$1; d++) {
                     dims[d] = int(pyarray->dimensions[d]);
                  }

                  $2 = dims;
                  $3 = (%(dtype)s*)pyarray->data;
                }
                %%typemap(freearg) (int %(n)s,int* %(ptv)s,%(dtype)s* %(array)s){
                    // deleting dims
                    delete $2;
                }
                """ % { 'n' : a[0] , 'ptv' : a[1], 'array' : a[2], 'dtype' : DATA_TYPE })
            elif len(a) == 2:
                # 1-dimensional arrays, i.e. vectors
                typemaps += reindent("""
                %%apply (int DIM1, %(dtype)s* INPLACE_ARRAY1) {(int %(n1)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'array' : a[1], 'dtype' : DATA_TYPE })
            elif len(a) == 3:
                # 2-dimensional arrays, i.e. matrices
                typemaps += reindent("""
                %%apply (int DIM1, int DIM2, %(dtype)s* INPLACE_ARRAY2) {(int %(n1)s, int %(n2)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'n2' : a[1], 'array' : a[2], 'dtype' : DATA_TYPE })
            else:
                # 3-dimensional arrays, i.e. tensors
                typemaps += reindent("""
                %%apply (int DIM1, int DIM2, int DIM3, %(dtype)s* INPLACE_ARRAY3) {(int %(n1)s, int %(n2)s, int %(n3)s, %(dtype)s* %(array)s)};
                """ % { 'n1' : a[0], 'n2' : a[1], 'n3' : a[2], 'array' : a[3], 'dtype' : DATA_TYPE})
            # end
        # end if
    # end for

    system_headers_code = mapstrings('#include <%s>', system_headers)
    local_headers_code  = mapstrings('#include "%s"', local_headers)
    wrap_headers_code1  = mapstrings('#include "%s"', wrap_headers)
    wrap_headers_code2  = mapstrings('%%include "%s"', wrap_headers)

    numpy_i_include = ''
    if arrays:
        numpy_i_include = r'%include "numpy.i"'

    # Do not reindent as SWIG interface code can also include Python code.
    interface_string = """%%module  %(modulename)s
//%%module (directors="1") %(modulename)s

//%%feature("director");

%%{
#include <iostream>
%(additional_definitions)s
%(system_headers_code)s
%(local_headers_code)s
%(wrap_headers_code1)s
%(code)s
%%}

//%%feature("autodoc", "1");
%(numpy_i_include)s

%%init%%{
%(init_code)s
%%}

%(additional_definitions)s
%(additional_declarations)s
%(wrap_headers_code2)s
//%(typemaps)s
%(code)s;

""" % locals()

    write_file(filename, interface_string)
    instant_debug("Done generating interface file.")

def write_setup(filename, modulename, csrcs, cppsrcs, local_headers, include_dirs, library_dirs, libraries, swig_include_dirs, swigargs, cppargs, lddargs):
    """Generate a setup.py file. Intended for internal library use."""
    instant_debug("Generating %s." % filename)

    swig_include_dirs.append(os.path.join(os.path.dirname(__file__), 'swig'))

    # Handle arguments
    swigfilename = "%s.i" % modulename
    wrapperfilename = "%s_wrap.cxx" % modulename

    # Treat C and C++ files in the same way for now
    cppsrcs = cppsrcs + csrcs + [wrapperfilename]

    swig_args = ""
    if swigargs:
        swig_args = " ".join(swigargs)

    compile_args = ""
    if cppargs:
        compile_args = ", extra_compile_args=%r" % cppargs

    link_args = ""
    if lddargs:
        link_args = ", extra_link_args=%r" % lddargs

    swig_include_dirs = " ".join("-I%s"%d for d in swig_include_dirs)
    if len(local_headers) > 0:
        swig_include_dirs += " -I.."

    py3 = "" if sys.version_info[0] < 3 else "-py3"

    # Generate code
    code = reindent("""
        import os
        from distutils.core import setup, Extension
        name = '%s'
        swig_cmd =r'%s -python %s %s %s %s'
        os.system(swig_cmd)
        sources = %s
        setup(name = '%s',
              ext_modules = [Extension('_' + '%s',
                             sources,
                             include_dirs=%s,
                             library_dirs=%s,
                             libraries=%s %s %s)])
        """ % (modulename, get_swig_binary(), py3, swig_include_dirs, swig_args, \
               swigfilename, cppsrcs, modulename, modulename, include_dirs, \
               library_dirs, libraries, compile_args, link_args))

    write_file(filename, code)
    instant_debug("Done writing setup.py file.")


def _test_write_interfacefile():
    modulename = "testmodule"
    code = "void foo() {}"
    init_code = "/* custom init code */"
    additional_definitions = "/* custom definitions */"
    additional_declarations = "/* custom declarations */"
    system_headers = ["system_header1.h", "system_header2.h"]
    local_headers = ["local_header1.h", "local_header2.h"]
    wrap_headers = ["wrap_header1.h", "wrap_header2.h"]
    arrays = [["length1", "array1"], ["dims", "lengths", "array2"]]

    write_interfacefile("%s.i" % modulename, modulename, code, init_code, \
                        additional_definitions, additional_declarations, \
                        system_headers, local_headers, wrap_headers, arrays)
    print("".join(io.open("%s.i" % modulename, encoding="utf8").readlines()))

def _test_write_setup():
    modulename = "testmodule"
    csrcs = ["csrc1.c", "csrc2.c"]
    cppsrcs = ["cppsrc1.cpp", "cppsrc2.cpp"]
    local_headers = ["local_header1.h", "local_header2.h"]
    include_dirs = ["includedir1", "includedir2"]
    library_dirs = ["librarydir1", "librarydir2"]
    libraries = ["lib1", "lib2"]
    swig_include_dirs = ["swigdir1", "swigdir2"],
    swigargs = ["-Swigarg1", "-Swigarg2"]
    cppargs = ["-cpparg1", "-cpparg2"]
    lddargs = ["-Lddarg1", "-Lddarg2"]

    write_setup("setup.py", modulename, csrcs, cppsrcs, local_headers, \
                include_dirs, library_dirs, libraries, swig_include_dirs, \
                swigargs, cppargs, lddargs)
    print("".join(io.open("setup.py", encoding="utf8").readlines()))

def unique(sequence):
    return list(set(sequence))


def find_vtk_classes(str):
    pattern = "vtk\w*"
    l = unique(re.findall(pattern, str))
    return l

def create_typemaps(classes):
    s = ""

    typemap_template = """
%%typemap(in) %(class_name)s * {
    vtkObjectBase* obj = vtkPythonGetPointerFromObject($input, "%(class_name)s");
    %(class_name)s * oobj = NULL;
    if (obj->IsA("%(class_name)s")) {
        oobj = %(class_name)s::SafeDownCast(obj);
        $1 = oobj;
    }
}

%%typemap(out) %(class_name)s * {
   $result = vtkPythonGetObjectFromPointer($1);
}

   """

    for cl in classes:
        s += typemap_template % { "class_name" : cl }

    return s


def generate_vtk_includes(classes):
    s = """
#include "vtkPythonUtil.h"
    """
    for cl in classes:
        s += """
#include \"%s.h\" """ % cl
    return s


def generate_interface_file_vtk(signature, code):

    interface_template =  """
%%module test
%%{

%(includes)s

%(code)s

%%}

%(typemaps)s

%(code)s

"""
    class_list = find_vtk_classes(code)
    includes = generate_vtk_includes(class_list)
    typemaps = create_typemaps(class_list)
    s = interface_template % { "typemaps" : typemaps, "code" : code, "includes" : includes }
    return s

def write_cmakefile(module_name, cmake_packages, csrcs, cppsrcs, local_headers, include_dirs, library_dirs, libraries, swig_include_dirs, swigargs, cppargs, lddargs):

    find_package_template = """
# Configuration for package %(package)s
FIND_PACKAGE(%(package)s REQUIRED)
IF(%(package)s_FOUND)
 INCLUDE(${%(PACKAGE)s_USE_FILE})
ENDIF(%(package)s_FOUND)
"""

    cmake_form = dict(module_name=module_name)

    cmake_form["python_executable"] = sys.executable

    cmake_form["extra_libraries"] = ";".join(libraries)
    cmake_form["extra_include_dirs"] = ";".join(include_dirs)
    cmake_form["extra_library_dirs"] = ";".join(library_dirs)
    cmake_form["extra_swig_include_dirs"] = " -I".join([" "] + swig_include_dirs)

    cmake_form["extra_swigargs"] = " ".join(swigargs)

    cmake_form["swig_executable"] =  "\n".join(\
        """if (DEFINED %(package)s_SWIG_EXECUTABLE)
  set(SWIG_EXECUTABLE ${%(package)s_SWIG_EXECUTABLE})
endif()
""" % dict(package=package.upper()) for package in cmake_packages)


    cmake_form["find_packages"] = "\n\n".join(find_package_template % \
                                              dict(package=package,
                                                   PACKAGE=package.upper())\
                                              for package in cmake_packages)
    cmake_form["packages_definitions"] = "\n".join(
        "${%s_CXX_DEFINITIONS}" % package.upper()
        for package in cmake_packages)

    cmake_form["packages_definitions"] += "\n"+"\n".join(
        "${%s_PYTHON_DEFINITIONS}" % package.upper()
        for package in cmake_packages)

    cmake_form["package_include_dirs"] = "\n".join(\
        "include_directories(${%(package)s_PYTHON_INCLUDE_DIRS} ${%(package)s_3RD_PARTY_INCLUDE_DIRS} ${${NAME}_SOURCE_DIR})" %
        dict(package=package.upper()) for package in cmake_packages)

    cmake_form["package_flags"] = "\n".join(\
        """set(CMAKE_EXE_LINKER_FLAGS \"${CMAKE_EXE_LINKER_FLAGS} ${%(package)s_LINK_FLAGS}\")
set(CMAKE_SHARED_LINKER_FLAGS \"${CMAKE_SHARED_LINKER_FLAGS} ${%(package)s_LINK_FLAGS}\")
""" %
        dict(package=package.upper()) for package in cmake_packages)

    cmake_form["package_swig_link_libraries"] = "\n".join(\
        """if (DEFINED %(package)s_LIBRARIES OR DEFINED %(package)s_3RD_PARTY_LIBRARIES OR DEFINED %(package)s_PYTHON_LIBRARIES)
  swig_link_libraries(${SWIG_MODULE_NAME} ${%(package)s_LIBRARIES} ${%(package)s_3RD_PARTY_LIBRARIES} ${%(package)s_PYTHON_LIBRARIES} ${EXTRA_SOURCE_LIB})
endif()""" %
        dict(package=package.upper()) for package in cmake_packages)

    cmake_form["package_python_definitions"] = "\n".join(\
        """if (DEFINED %(package)s_PYTHON_DEFINITIONS)
  add_definitions(${%(package)s_PYTHON_DEFINITIONS})
endif()""" %
        dict(package=package.upper()) for package in cmake_packages)

    cppsrcs.extend(csrcs)
    if len(cppsrcs) > 0:
        cmake_form["extra_sources_files"] = "set(SOURCE_FILES %s) " %  " ".join(cppsrcs)
    else:
        cmake_form["extra_sources_files"] = "set(SOURCE_FILES)"

    if cppargs:
        cmake_form["cppargs"] = "set(CMAKE_CXX_FLAGS \"${CMAKE_CXX_FLAGS} %s\")" % \
                                (" ".join(cppargs))
    else:
        cmake_form["cppargs"] = ""

    if lddargs:
        cmake_form["lddargs"] = "set(CMAKE_EXE_LINKER_FLAGS \""\
                                "${CMAKE_EXE_LINKER_FLAGS} %s\")" % (" ".join(lddargs))
    else:
        cmake_form["lddargs"] = ""

    cmake_template = """
cmake_minimum_required(VERSION 2.6.0)

set (NAME %(module_name)s)

PROJECT(${NAME})

set(PYTHON_EXECUTABLE %(python_executable)s)

%(find_packages)s

%(cppargs)s
%(lddargs)s

%(swig_executable)s

find_package(SWIG REQUIRED)
include(${SWIG_USE_FILE})

set(SWIG_MODULE_NAME ${NAME})
set(CMAKE_SWIG_FLAGS
  -module ${SWIG_MODULE_NAME}
  -shadow
  -modern
  -modernargs
  -fastdispatch
  -fvirtual
  -nosafecstrings
  -noproxydel
  -fastproxy
  -fastinit
  -fastunpack
  -fastquery
  -nobuildnone
%(packages_definitions)s
%(extra_swigargs)s
%(extra_swig_include_dirs)s
  )

set(CMAKE_SWIG_OUTDIR ${CMAKE_CURRENT_BINARY_DIR})

set(SWIG_SOURCES ${NAME}.i)

set_source_files_properties(${SWIG_SOURCES} PROPERTIES CPLUSPLUS ON)

set(EXTRA_INCLUDE_DIRS \"%(extra_include_dirs)s\")
if(EXTRA_INCLUDE_DIRS)
  include_directories(${EXTRA_INCLUDE_DIRS})
endif()
%(package_include_dirs)s

%(package_flags)s

set(EXTRA_LIBRARY_DIRS \"%(extra_library_dirs)s\")
if(EXTRA_LIBRARY_DIRS)
  link_directories(${EXTRA_LIBRARY_DIRS})
endif()

%(extra_sources_files)s

%(package_python_definitions)s

# Work-around for bug in CMake 3.0.0 (see
# http://www.cmake.org/Bug/view.php?id=14990)
set(SWIG_MODULE_NAME_ORIG "${SWIG_MODULE_NAME}")
if (${CMAKE_VERSION} MATCHES "3.0.0")
  set(SWIG_MODULE_NAME "_${SWIG_MODULE_NAME}")
endif()

swig_add_module(${SWIG_MODULE_NAME} python ${SWIG_SOURCES})

set(EXTRA_LIBRARIES %(extra_libraries)s)
if(SOURCE_FILES)
  set(CMAKE_CXX_FLAGS \"${CMAKE_CXX_FLAGS} -fpic\")
  add_library(source_file_lib
    STATIC
    ${SOURCE_FILES})
  set(EXTRA_LIBRARIES \"source_file_lib;${EXTRA_LIBRARIES}\")
endif()

if(EXTRA_LIBRARIES)
  string(STRIP \"${EXTRA_LIBRARIES}\" EXTRA_LIBRARIES)
  swig_link_libraries(${SWIG_MODULE_NAME} ${EXTRA_LIBRARIES})
endif()

%(package_swig_link_libraries)s

""" % cmake_form

    filename = "CMakeLists.txt"
    write_file(filename, cmake_template)

def write_itk_cmakefile(name):
    file_template = """
cmake_minimum_required(VERSION 2.6.0)

# This project is designed to be built outside the Insight source tree.
PROJECT(%(name)%s)

# Find ITK.
FIND_PACKAGE(ITK REQUIRED)
IF(ITK_FOUND)
  INCLUDE(${ITK_USE_FILE})
ENDIF(ITK_FOUND)

# Find VTK.
FIND_PACKAGE(VTK REQUIRED)
IF(VTK_FOUND)
  INCLUDE(${VTK_USE_FILE})
ENDIF(VTK_FOUND)

find_package(SWIG REQUIRED)
include(${SWIG_USE_FILE})

set(SWIG_MODULE_NAME %(name)s)
set(CMAKE_SWIG_FLAGS
  -module ${SWIG_MODULE_NAME}
  -shadow
  -modern
  -modernargs
  -fastdispatch
  -fvirtual
  -nosafecstrings
  -noproxydel
  -fastproxy
  -fastinit
  -fastunpack
  -fastquery
  -nobuildnone
  -Iinclude/swig
  )

set(CMAKE_SWIG_OUTDIR ${CMAKE_CURRENT_BINARY_DIR})

set(SWIG_SOURCES %(name)s.i)

set_source_files_properties(${SWIG_SOURCES} PROPERTIES CPLUSPLUS ON)

include_directories(${PYTHON_INCLUDE_PATH} ${%(name)s_SOURCE_DIR})

set(VTK_LIBS ITKCommon vtkCommon vtkImaging vtkIO vtkFiltering vtkRendering vtkGraphics vtkCommonPythonD vtkFilteringPythonD)

# Work-around for bug in CMake 3.0.0 (see
# http://www.cmake.org/Bug/view.php?id=14990)
set(SWIG_MODULE_NAME_ORIG "${SWIG_MODULE_NAME}")
if (${CMAKE_VERSION} MATCHES "3.0.0")
  set(SWIG_MODULE_NAME "_${SWIG_MODULE_NAME}")
endif()

swig_add_module(${SWIG_MODULE_NAME} python ${SWIG_SOURCES})

swig_link_libraries(${SWIG_MODULE_NAME} ${PYTHON_LIBRARIES} ${VTK_LIBS})


    """ % { "name" : name }

    with io.open("CMakeLists.txt", 'w', encoding="utf8") as f:
        f.write(file_template)


def write_vmtk_cmakefile(name):
    file_template = """
cmake_minimum_required(VERSION 2.6.0)

# This project is designed to be built outside the Insight source tree.
PROJECT(%(name)%s)

# Find ITK.
FIND_PACKAGE(ITK REQUIRED)
IF(ITK_FOUND)
  INCLUDE(${ITK_USE_FILE})
ENDIF(ITK_FOUND)

# Find VTK.
FIND_PACKAGE(VTK REQUIRED)
IF(VTK_FOUND)
  INCLUDE(${VTK_USE_FILE})
ENDIF(VTK_FOUND)

# Find VMTK.
#FIND_PACKAGE(VMTK REQUIRED)
#IF(VMTK_FOUND)
#  INCLUDE(${VMTK_USE_FILE})
#ENDIF(ITK_FOUND)

find_package(SWIG REQUIRED)
include(${SWIG_USE_FILE})

set(SWIG_MODULE_NAME %(name)s)
set(CMAKE_SWIG_FLAGS
  -module ${SWIG_MODULE_NAME}
  -shadow
  -modern
  -modernargs
  -fastdispatch
  -fvirtual
  -nosafecstrings
  -noproxydel
  -fastproxy
  -fastinit
  -fastunpack
  -fastquery
  -nobuildnone
  -Iinclude/swig
  )

set(CMAKE_SWIG_OUTDIR ${CMAKE_CURRENT_BINARY_DIR})

set(SWIG_SOURCES %(name)s.i)

set_source_files_properties(${SWIG_SOURCES} PROPERTIES CPLUSPLUS ON)

include_directories(${PYTHON_INCLUDE_PATH} ${%(name)s_SOURCE_DIR} /usr/local/include/vmtk)
link_directories(/usr/local/lib/vmtk .)

set(VTK_LIBS ITKCommon vtkCommon vtkImaging vtkIO vtkFiltering vtkRendering vtkGraphics vtkCommonPythonD vtkFilteringPythonD)
set(VMTK_LIBS vtkvmtkCommonPythonD vtkvmtkITKPythonD vtkvmtkCommon vtkvmtkITK vtkvmtkComputationalGeometryPythonD vtkvmtkMiscPythonD vtkvmtkComputationalGeometry vtkvmtkMisc vtkvmtkDifferentialGeometryPythonD vtkvmtkSegmentationPythonD vtkvmtkDifferentialGeometry vtkvmtkSegmentation vtkvmtkIOPythonD)

# Work-around for bug in CMake 3.0.0 (see
# http://www.cmake.org/Bug/view.php?id=14990)
set(SWIG_MODULE_NAME_ORIG "${SWIG_MODULE_NAME}")
if (${CMAKE_VERSION} MATCHES "3.0.0")
  set(SWIG_MODULE_NAME "_${SWIG_MODULE_NAME}")
endif()

swig_add_module(${SWIG_MODULE_NAME} python ${SWIG_SOURCES})

swig_link_libraries(${SWIG_MODULE_NAME} ${PYTHON_LIBRARIES} ${VTK_LIBS} ${VMTK_LIBS})


    """ % { "name" : name }

    with io.open("CMakeLists.txt", 'w', encoding="utf8") as f:
        f.write(file_template)


def write_vtk_interface_file(signature, code):
    filename = signature
    ifile = filename + ".i"
    ifile_code = generate_interface_file_vtk(signature, code)
    with io.open(ifile, 'w', encoding="utf8") as iff:
        iff.write(ifile_code)


if __name__ == "__main__":
    _test_write_interfacefile()
    print("\n"*3)
    _test_write_setup()




from instant import inline_vtk
import vtk 

c_code1 = """
vtkObject* test1(char* filename){
  std::cout <<" Reading grid file  "<<std::endl; 
  vtkXMLUnstructuredGridReader* reader1 = vtkXMLUnstructuredGridReader::New();
  reader1->SetFileName(filename); 
  reader1->Update();
  vtkUnstructuredGrid* grid;
  grid= reader1->GetOutput();
  std::cout << "Number of cells "<<grid->GetNumberOfCells () <<std::endl; 
  return grid; 
}
"""

c_code2 = """
void test2(vtkUnstructuredGrid* grid){
  std::cout << "number of cells "<<grid->GetNumberOfCells () <<std::endl; 
  vtkCell* cell; 
  vtkPoints* points; 
  double x[3]; 
  for (int i=0; i<grid->GetNumberOfCells (); i++) { 
    cell = grid->GetCell (i);
    std::cout <<"cell no. "<<i <<" cell type "<< cell->GetCellType () <<std::endl; 
    points = cell->GetPoints ();
    std::cout <<"points->GetNumberOfPoints() "<< points->GetNumberOfPoints() <<std::endl;  
    for (int d=0; d<cell->GetNumberOfPoints(); d++){
	points->GetPoint(d, x); 
        std::cout <<" x " <<x[0]<<" "<<x[1]<<" "<<x[2]<<std::endl;  
    }
    vtkIdList* ids = cell->GetPointIds(); 
    for (int d=0; d<ids->GetNumberOfIds(); d++){
        std::cout <<" ids " <<ids->GetId(d) <<std::endl;  
    }
  }
}
"""


func1 = inline_vtk(c_code1, cache_dir="test_vtk")
func2 = inline_vtk(c_code2, cache_dir="test_vtk")

filename = "u2.vtu"
filename = "u000000.vtu"

mesh = func1(filename)
func2(mesh)





from __future__ import print_function
import pytest
import instant
from instant import build_module, import_module

def test_sum():

    sig = "((instant unittest test16.py))"

    # Trying to import module
    module = import_module(sig, cache_dir="test_cache")
    if module is None:
        print("Defining code")
        c_code = """
        class Sum {
        public:
          virtual double sum(double a, double b){
          return a+b;
        }
      };

      double use_Sum(Sum& sum, double a, double b) {
        return sum.sum(a,b);
      }
        """
        print("Compiling code")
        module = build_module(code=c_code, signature=sig, cache_dir="test_cache")

    # Testing module
    Sum = module.Sum
    use_Sum = module.use_Sum

    sum = Sum()
    a = 3.7
    b = 4.8
    c = use_Sum(sum, a, b)
    print("The sum of %g and %g is %g"% (a, b, c))

    class Sub(Sum):
        def __init__(self):
            Sum.__init__(self)

        def sum(self, a, b):
            print("sub")
            return a-b;

    sub = Sub()
    a = 3.7
    b = 4.8
    c = use_Sum(sub, a, b)
    print("The sub of %g and %g is %g"% (a, b, c))



from __future__ import print_function
import pytest
from instant import inline

def test_inline():
    add_func = inline("double add(double a, double b){ return a+b; }",
                      cache_dir="test_cache")
    assert add_func(3, 4.5) == 7.5



from __future__ import print_function

from instant import inline_vmtk 
import vmtk
from vmtk import vtkvmtk

code = """
void test(vtkvmtkDoubleVector* a) {
  for (int i=0; i<a->GetNumberOfElements(); i++) {
    a->SetElement(i,i); 
  }
}
"""



func = inline_vmtk(code, cache_dir="test_vmtk")

v = vtkvmtk.vtkvmtkDoubleVector()
v.Allocate(12, 1)
print("norm of v ", v.ComputeNorm())
func(v)
print("norm of v after test ", v.ComputeNorm())




from __future__ import print_function
import pytest
import numpy as N
import time
from instant import build_module
import sys
from functools import reduce


c_code = """
void func(int n1, double* array1, int n2, double* array2){
    double a;
    if ( n1 == n2 ) {
        for (int i=0; i<n1; i++) {
            a = array1[i];
            array2[i] = sin(a) + cos(a) + tan(a);
        }
    } else {
        printf("The arrays should have the same size.");
    }
}
"""


def test_module():
    # Guess arrayobject is either in sys.prefix or /usr/local

    test7_ext = build_module(code=c_code, system_headers=["numpy/arrayobject.h"], cppargs='-g',
                             include_dirs=[N.get_include()],
                             init_code='import_array();', modulename='test7_ext',
                             arrays = [['n1', 'array1'], ['n2', 'array2']])

    seed = 10000000.0

    a = N.arange(seed)
    t1 = time.time()
    b = N.sin(a) + N.cos(a) + N.tan(a)
    t2 = time.time()
    print("With NumPy: ", t2-t1, "seconds")

    from test7_ext import func
    c = N.arange(seed)
    t1 = time.time()
    func(a, c)
    t2 = time.time()
    print("With instant: ", t2-t1, "seconds")

    t1 = time.time()
    d = N.sin(a)
    d += N.cos(a)
    d += N.tan(a)
    t2 = time.time()
    print("With NumPy inplace aritmetic: ", t2-t1, "seconds")

    difference = abs(b - c)
    sum = reduce( lambda a, b: a+b, difference)
    assert abs(sum) < 1.0e-12



from __future__ import print_function

from instant import inline_with_numpy
from numpy import *
import time


def time_loop2(p, Q, A, B, dt, N, p0): 
    p[0] = p0 
    for i in range(1, N): 
        p[i] = p[i-1] + dt*(B*Q[i] - A*p[i-1])

c_code = r'''
void time_loop(int n, double* p, 
             int m, double* Q, 
             double A, double B,  
             double dt, int N, double p0){

    if ( n != m ) {
        printf("n and m should be equal");  
        return; 
    }
    if ( n != N ) {
        printf("n and N should be equal");  
        return; 
    }
    #pragma omp parallel
    {
    int id;
    id = omp_get_thread_num(); 
    printf("Thread %d\n", id);

    p[0] = p0; 
    #pragma omp for
    for (int i=1; i<n; i++) { 
        p[i] = p[i-1] + dt*(B*Q[i] - A*p[i-1]);
    }
    }
}
'''

N = 20000000
time_loop = inline_with_numpy(c_code, arrays = [['n', 'p'], ['m', 'Q']],
                              cppargs=['-fopenmp'], 
                              lddargs=['-lgomp'],
                              system_headers=['omp.h'])
p = zeros(N)
T = 20.0
Q = sin(arange(0, T, T/N))+1   

t1 = time.time()
t3 = time.clock()
time_loop(p, Q, 1.0, 1.0, 1.0/N, N, 1.0)
t2 = time.time()
t4 = time.clock()

print('With instant and OpenMP:', t4-t3, 'seconds process time')
print('With instant and OpenMP:', t2-t1, 'seconds process time')

#p2 = zeros(N)
#t1 = time.time()
#time_loop2(p2, Q, 1.0, 1.0, 1.0/N, N, 1.0)
#t2 = time.time()
#print 'With Python:',t2-t1,'seconds'
#
#print 'The max difference between p and p2 is ', max(abs(p - p2)) 


tt = arange(0, 1, 1.0/N) 

a = """
try: 
    import pylab
    pylab.plot(tt, p)
    pylab.plot(tt, Q)
    pylab.xlabel('time ')
    pylab.ylabel('pressure ')
    pylab.title('Pressure ')
    #pylab.savefig('pressure_plot')
    pylab.show()
except: 
    print "To get a plot of the solution install pylab"
"""








from __future__ import print_function
import pytest
import time
from instant import build_module, import_module


def test_timing():
    #_t = None
    def tic():
        global _t
        _t = -time.time()

    def toc(msg=""):
        t = time.time() + _t
        print("t = %f  (%s)" % (t, msg))
        return t

    c_code = """
double sum(double a, double b)
{
  return a+b;
}
    """

    class Sig:
        def __init__(self, sig):
            self.sig = sig

        def signature(self):
            time.sleep(1.0)
            return self.sig

        def __hash__(self):
            time.sleep(0.5)
            return hash(self.sig)

        def __cmp__(self, other):
            if isinstance(other, Sig):
                return cmp(self.sig, other.sig)
            return -1

    sig = Sig("((test18.py signature))")
    cache_dir = "test_cache"

    # Time a few builds
    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t1 = toc("first build")

    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t2 = toc("second build")

    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t3 = toc("third build")

    # Time importing
    tic()
    module = import_module(sig, cache_dir)
    assert module is not None
    t4 = toc("first import")

    tic()
    module = import_module(sig, cache_dir)
    assert module is not None
    t5 = toc("second import")

    assert t1 > 1
    assert t2 < 1 and t2 > 0.4
    assert t3 < 1 and t3 > 0.4
    assert t4 < 1 and t4 > 0.4
    assert t5 < 1 and t5 > 0.4



from __future__ import print_function
import pytest
import shutil
import time
import os, sys
from instant import build_module, import_module

def test_build():

    #_t = None
    def tic():
        global _t
        _t = -time.time()

    def toc(msg=""):
        t = time.time() + _t
        print("t = %f  (%s)" % (t, msg))
        return t

    c_code = """
double sum(double a, double b)
{
  return a+b;
}
    """

    class Sig:
        def __init__(self, sig):
            self.sig = sig

        def signature(self):
            time.sleep(1.0)
            return self.sig

        def __hash__(self):
            time.sleep(0.5)
            return hash(self.sig)

        def __cmp__(self, other):
            if isinstance(other, Sig):
                return cmp(self.sig, other.sig)
            return -1

    modulename = "test19_ext"
    cache_dir = "test19_cache"
    shutil.rmtree(cache_dir, ignore_errors=True)
    shutil.rmtree(modulename, ignore_errors=True)

    # Build and rebuild with explicit modulename
    tic()
    module = build_module(code=c_code, modulename=modulename,
                          cache_dir=cache_dir)
    assert module is not None
    t1 = toc("(1) With modulename")

    tic()
    module = build_module(code=c_code, modulename=modulename,
                          cache_dir=cache_dir)
    assert module is not None
    t2 = toc("(2) With modulename")
    assert t1 > t2

    # Try importing module in a separate python process
    python_interp = sys.executable
    cmd = python_interp + ' -c "import %s"' % modulename
    print(cmd)
    stat = os.system(cmd)
    assert stat == 0 # a

    # Build and rebuild with a valid filename as signature
    sig = "test19_signature_module"
    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t1 = toc("(1) With signature")

    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t2 = toc("(2) With signature")
    assert t1 > t2

    tic()
    module = import_module(sig, cache_dir)
    assert module is not None
    t3 = toc("(3) import_module")
    assert t1 > t3

    # Try importing module in a separate python process
    python_interp = sys.executable
    cmd = python_interp + ' -c "import instant; assert instant.import_module(\'%s\', \'%s\') is not None"' % (sig, cache_dir)
    print(cmd)
    stat = os.system(cmd)
    assert stat == 0 # b

    # Build and rebuild with generic signature string
    sig = "((test19_signature_module))"
    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t1 = toc("(1) With signature")

    tic()
    module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    assert module is not None
    t2 = toc("(2) With signature")
    assert t1 > t2

    tic()
    module = import_module(sig, cache_dir)
    assert module is not None
    t3 = toc("(3) import_module")
    assert t1 > t3

    # Try importing module in a separate python process
    python_interp = sys.executable
    cmd = python_interp + ' -c "import instant; assert instant.import_module(\'%s\', \'%s\') is not None"' % (sig, cache_dir)
    print(cmd)
    stat = os.system(cmd)
    assert stat == 0 # c

    print("Skipping unit test, see https://bugs.launchpad.net/instant/+bug/518389")
    # Build and rebuild with generic signature object
    #sig = Sig("((test19_signature_module))")
    #tic()
    #module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    #assert module is not None
    #t1 = toc("(1) With signature")
    #tic()
    #module = build_module(code=c_code, signature=sig, cache_dir=cache_dir)
    #assert module is not None
    #t2 = toc("(2) With signature")
    #assert t1 > t2
    #tic()
    #module = import_module(sig, cache_dir)
    #assert module is not None
    #t3 = toc("(3) import_module")
    #assert t1 > t3

    # Build and rebuild without modulename or signature
    tic()
    module = build_module(code=c_code, cache_dir=cache_dir)
    assert module is not None
    t1 = toc("(1) Without modulename or signature")

    tic()
    module = build_module(code=c_code, cache_dir=cache_dir)
    assert module is not None
    t2 = toc("(2) Without modulename or signature")
    assert t1 > t2



from __future__ import print_function
import sys, os, threading
from instant import inline

def compile_stuff(i):
    c_code = """
    double sum(int a, int b)
    { return a + b; }
    // Code number %d
    """ % (i % 2)
    func = inline(c_code, cache_dir="test_locking_cache")
    s = func(i, 3)
    assert s == (i+3)
    print("In run %d, result is %d as expected." % (i, s))

if __name__ == "__main__":
    cmd = sys.argv[0]
    args = sys.argv[1:]

    if args:
        print("Compiling with args =", args)
        i, = args
        compile_stuff(int(i))

    else:
        print("Starting new processes to compile.")
        def run(i):
            os.system("python %s %d" % (cmd, i))
        threads = []
        for i in range(10):
            threads.append( threading.Thread(target=run, args=(i,)) )
        for t in threads:
            t.start()
        for t in threads:
            t.join()



from __future__ import print_function
import pytest
import time
#from math import exp, sin, cos

def test_2():

    try:
        from sympy import Symbol, exp, sin, cos
    except:
        print("You need sympy for this test")
        return

    x = Symbol('x')
    pi = 3.14
    f =  -3*x**7 - 2*x**3 + 2*exp(x**2) + x**12*(2*exp(2*x) - pi*sin(x)**((-1) + pi)*cos(x)) + 4*(pi**5 + x**5 + 5*pi*x**4 + 5*x*pi**4 + 10*pi**2*x**3 + 10*pi**3*x**2)*exp(123 + x - x**5 + 2*x**4)

    a = []
    t0 = time.time()
    for i in range(0, 1000):
        xx = i/1000.0
        y = f.subs(x, xx)
        a.append(y)

    t1 = time.time()

    print("Elapsed time with sympy", t1-t0)


def test_3():
    try:
        from sympy import Symbol, exp, sin, cos
    except:
        print("You need sympy for this test")
        return

    x = Symbol('x')
    pi = 3.14

    f = lambda x : -3*x**7 - 2*x**3 + 2*exp(x**2) + x**12*(2*exp(2*x) - pi*sin(x)**((-1) + pi)*cos(x))  \
      + 4*(pi**5 + x**5 + 5*pi*x**4 + 5*x*pi**4 + 10*pi**2*x**3 + 10*pi**3*x**2)*exp(123 + x - x**5 + 2*x**4)

    a = []
    t0 = time.time()
    for i in range(0, 1000):
        xx = i/1000.0
        y = f(xx)
        a.append(y)

    t1 = time.time()

    print("Elapsed time with lambda and math", t1-t0)



"""This module contains the inline* functions, which allows easy inlining of C/C++ functions."""

# Copyright (C) 2008-2010 Kent-Andre Mardal
# Copyright (C) 2008-2010 Martin Sandve Alnes
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

from .output import instant_assert, instant_warning, instant_error
from .build import build_module, build_module_vtk, build_module_vmtk


def get_func_name(c_code):
    # TODO: Something more robust? Regexp?
    try:
        func = c_code[:c_code.index('(')]
        ret, func_name = func.split()
    except:
        instant_error("Failed to extract function name from c_code.")
    return func_name


def inline(c_code, **kwargs):
    """This is a short wrapper around the build_module function in instant.

    It creates a module given that
    the input is a valid C function. It is only possible
    to inline one C function each time.

    Usage:

    >>> from instant import inline
    >>> add_func = inline("double add(double a, double b){ return a+b; }")
    >>> print "The sum of 3 and 4.5 is ", add_func(3, 4.5)
    """
    instant_assert("code" not in kwargs, "Cannot specify code twice.")
    kwargs["code"] = c_code
    func_name = get_func_name(c_code)
    module = build_module(**kwargs)
    if hasattr(module, func_name):
        return getattr(module, func_name)
    else:
        instant_warning("Didn't find function '%s', returning module." % func_name)
    return module


def inline_module(c_code, **kwargs):
    """This is a short wrapper around the build_module function in instant.

    It creates a module given that
    the input is a valid C function. It is only possible
    to inline one C function each time.

    Usage:

    >>> from instant import inline
    >>> add_func = inline("double add(double a, double b){ return a+b; }")
    >>> print "The sum of 3 and 4.5 is ", add_func(3, 4.5)
    """
    instant_assert("code" not in kwargs, "Cannot specify code twice.")
    kwargs["code"] = c_code
    module = build_module(**kwargs)
    return module



def inline_with_numpy(c_code, **kwargs):
    '''This is a short wrapper around the build_module function in instant.

    It creates a module given that
    the input is a valid C function. It is only possible
    to inline one C function each time. The difference between
    this function and the inline function is that C-arrays can be used.
    The following example illustrates that.

    Usage:

    >>> import numpy
    >>> import time
    >>> from instant import inline_with_numpy
    >>> c_code = """
        double sum (int n1, double* array1){
            double tmp = 0.0;
            for (int i=0; i<n1; i++) {
                tmp += array1[i];
            }
            return tmp;
        }
        """
    >>> sum_func = inline_with_numpy(c_code,  arrays = [['n1', 'array1']])
    >>> a = numpy.arange(10000000); a = numpy.sin(a)
    >>> sum_func(a)
    '''

    import numpy
    instant_assert("code" not in kwargs, "Cannot specify code twice.")
    kwargs["code"] = c_code
    kwargs["init_code"]      = kwargs.get("init_code", "")      + "\nimport_array();\n"
    kwargs["system_headers"] = kwargs.get("system_headers", []) + ["numpy/arrayobject.h"]
    kwargs["include_dirs"]   = kwargs.get("include_dirs", [])   + ["%s" %numpy.get_include()]
    func_name = get_func_name(c_code)
    module = build_module(**kwargs)
    if hasattr(module, func_name):
        return getattr(module, func_name)
    else:
        instant_warning("Didn't find function '%s', returning module." % func_name)
    return module

def inline_module_with_numpy(c_code, **kwargs):
    '''This is a short wrapper around the build_module function in instant.

    It creates a module given that
    the input is a valid C function. It is only possible
    to inline one C function each time. The difference between
    this function and the inline function is that C-arrays can be used.
    The following example illustrates that.

    Usage:

    >>> import numpy
    >>> import time
    >>> from instant import inline_with_numpy
    >>> c_code = """
        double sum (int n1, double* array1){
            double tmp = 0.0;
            for (int i=0; i<n1; i++) {
                tmp += array1[i];
            }
            return tmp;
        }
        """
    >>> sum_func = inline_with_numpy(c_code,  arrays = [['n1', 'array1']])
    >>> a = numpy.arange(10000000); a = numpy.sin(a)
    >>> sum_func(a)
    '''
    import numpy
    instant_assert("code" not in kwargs, "Cannot specify code twice.")
    kwargs["code"] = c_code
    kwargs["init_code"]      = kwargs.get("init_code", "")      + "\nimport_array();\n"
    kwargs["system_headers"] = kwargs.get("system_headers", []) + ["numpy/arrayobject.h"]
    kwargs["include_dirs"]   = kwargs.get("include_dirs", [])   + ["%s" % numpy.get_include()]
    module = build_module(**kwargs)
    return module


def inline_vtk(c_code, cache_dir=None):

    module = build_module_vtk(c_code)
    func_name = get_func_name(c_code)
    if hasattr(module, func_name):
        return getattr(module, func_name)
    else:
        instant_warning("Didn't find function '%s', returning module." % func_name)
    return module

def inline_vmtk(c_code, cache_dir=None):

    module = build_module_vmtk(c_code)
    func_name = get_func_name(c_code)
    if hasattr(module, func_name):
        return getattr(module, func_name)
    else:
        instant_warning("Didn't find function '%s', returning module." % func_name)
    return module



from __future__ import print_function
import pytest
from instant import build_module

c_code = """
class Sum {
public:
  virtual double sum(double a, double b){
    return a+b;
  }
};


double use_Sum(Sum& sum, double a, double b) {
  return sum.sum(a,b);
}
"""

def test_build_module():
    test8_ext = build_module(code=c_code, modulename='test8_ext')

    from test8_ext import Sum, use_Sum
    sum = Sum()
    a = 3.7
    b = 4.8
    c = use_Sum(sum, a, b)
    assert c == 8.5


    class Sub(Sum):
        def __init__(self):
            Sum.__init__(self)

        def sum(self, a, b):
            print("sub")
            return a-b;

    sub = Sub()
    a = 3.7
    b = 4.8
    c = use_Sum(sub, a, b)
    assert c == 8.5



from __future__ import print_function
import pytest
from instant import build_module
import numpy, sys
from functools import reduce

a = numpy.arange(10000000)
a = numpy.sin(a)
b = numpy.arange(10000000)
b = numpy.cos(b)

s = """
PyObject* add(PyObject* a_, PyObject* b_){
  /*
  various checks
  */
  PyArrayObject* a=(PyArrayObject*) a_;
  PyArrayObject* b=(PyArrayObject*) b_;

  int n = a->dimensions[0];

  npy_intp dims[1];
  dims[0] = n;
  PyArrayObject* ret;
  ret = (PyArrayObject*) PyArray_SimpleNew(1, dims, PyArray_DOUBLE);

  int i;
  double aj;
  double bj;
  double *retj;
  for (i=0; i < n; i++) {
    retj = (double*)(ret->data+ret->strides[0]*i);
    aj = *(double *)(a->data+ a->strides[0]*i);
    bj = *(double *)(b->data+ b->strides[0]*i);
    *retj = aj + bj;
  }
return PyArray_Return(ret);
}
"""

def test_build_module():

    test4_ext = build_module(code=s, system_headers=["numpy/arrayobject.h"],
                             include_dirs=[numpy.get_include()],
                             init_code='import_array();', modulename="test4_ext")

    import time

    t1 = time.time()
    d = test4_ext.add(a, b)
    t2 = time.time()

    print('With instant:', t2 - t1, 'seconds')

    t1 = time.time()
    c = a + b
    t2 = time.time()

    print('With numpy:   ', t2 - t1, 'seconds')

    difference = abs(c - d)
    sum = reduce( lambda a, b: a + b, difference)
    assert sum < 1.0e-12



#!/usr/bin/env python

import sys, platform
from os.path import join, split, pardir

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.version_info < (2, 7):
    print("Python 2.7 or higher required, please upgrade.")
    sys.exit(1)

scripts = [join("scripts", "instant-clean"),
           join("scripts", "instant-showcache")]

if platform.system() == "Windows" or "bdist_wininst" in sys.argv:
    # In the Windows command prompt we can't execute Python scripts
    # without a .py extension. A solution is to create batch files
    # that runs the different scripts.
    batch_files = []
    for script in scripts:
        batch_file = script + ".bat"
        f = open(batch_file, "w")
        f.write('python "%%~dp0\%s" %%*\n' % split(script)[1])
        f.close()
        batch_files.append(batch_file)
    scripts.extend(batch_files)

version = "2017.1.0"

url = "https://bitbucket.org/fenics-project/instant/"
tarball = None
if not 'dev' in version:
    tarball = url + "downloads/instant-%s.tar.gz" % version

CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
Intended Audience :: Science/Research
License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)
Operating System :: POSIX
Operating System :: POSIX :: Linux
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Programming Language :: Python
Programming Language :: Python :: 2
Programming Language :: Python :: 2.7
Programming Language :: Python :: 3
Programming Language :: Python :: 3.4
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Topic :: Scientific/Engineering :: Mathematics
Topic :: Software Development :: Libraries :: Python Modules
"""

requires = ["numpy", "six"]
if sys.version_info[0] == 2:
    requires.append("subprocess32")

setup(name="instant",
      version=version,
      description="Instant Inlining of C/C++ in Python",
      author="Magne Westlie, Kent-Andre Mardal, Martin Sandve Alnes and Ilmar M. Wilbers",
      author_email="fenics-dev@googlegroups.com",
      classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
      url=url,
      download_url=tarball,
      packages=['instant'],
      package_dir={'instant': 'instant'},
      package_data={'': [join('swig', 'numpy.i')]},
      scripts=scripts,
      install_requires=requires,
      data_files=[(join("share", "man", "man1"),
                   [join("doc", "man", "man1", "instant-clean.1.gz"),
                    join("doc", "man", "man1", "instant-showcache.1.gz")])]
      )



from __future__ import print_function
import pytest
from instant import build_module
import numpy
import sys
import time
from functools import reduce


c_code = """
/* add function for vectors with all safety checks removed ..*/
void add(int n1, double* array1, int n2, double* array2, int n3, double* array3){
  if ( n1 == n2 && n1 == n3 ) {
     for (int i=0; i<n1; i++) {
        array3[i] = array1[i] + array2[i];
     }
  }
  else {
    printf("The arrays should have the same size.");
  }

}
"""

def test_buil_module():
    test5_ext = build_module(code=c_code, system_headers=["numpy/arrayobject.h"],
                             cppargs=['-pg'], lddargs=['-pg'],
                             include_dirs=[numpy.get_include()],
                             init_code='import_array();', modulename='test5_ext',
                             arrays = [['n1', 'array1'], ['n2', 'array2'], ['n3', 'array3']])

    from test5_ext import add
    add = test5_ext.add
    a = numpy.arange(10000000); a = numpy.sin(a)
    b = numpy.arange(10000000); b = numpy.cos(b)
    c = numpy.arange(10000000); c = numpy.cos(c)
    d = numpy.arange(10000000); d = numpy.cos(d)

    t1 = time.time()
    add(a, b, c)
    t2 = time.time()
    print('With instant:', t2-t1, 'seconds')

    t1 = time.time()
    numpy.add(a, b, d)
    t2 = time.time()
    print('With numpy:   ', t2-t1, 'seconds')

    difference = abs(d - c)
    sum = reduce( lambda a, b: a+b, difference)
    assert abs(sum) < 1.0e-12



from __future__ import print_function
# Zombie model as described by 
# WHEN ZOMBIES ATTACK !: MATHEMATICAL MODELLING OF AN OUTBREAK OF ZOMBIE INFECTION
# by   Philip Munz, Ioan Hudea, Joe Imad, Robert J. Smith.
# In: Infectious Disease Modelling Research Progress, 
# Editors: J.M. Tchuenche and C. Chiyaka, pp. 133-150 2009 Nova Science Publishers, Inc.


from instant import inline_with_numpy
from numpy import *





    
c_code = """
void time_loop(int nS, double* S, 
               int nI, double* I, 
               int nZ, double* Z, 
               int nR, double* R, 
               double PI, double alpha, double beta, double delta, double rho, double zeta,  
               double dt, int N, double S0, double I0, double Z0, double R0)
{
    if ( nS != nI || nS != nZ || nS != nR || nS != N ) {
        printf("Arrays must be of same size!");  
        return; 
    }

    S[0] = S0; 
    I[0] = I0; 
    Z[0] = Z0; 
    R[0] = R0; 
    for (int i=0; i<N-1; i++) { 
        S[i+1] = S[i] + dt*(PI - beta*S[i]*Z[i] - delta*S[i]);
        I[i+1] = I[i] + dt*(beta*S[i]*Z[i] - (rho + delta)*I[i]); 
        Z[i+1] = Z[i] + dt*(rho*I[i] + zeta*R[i] - alpha*S[i]*Z[i]); 
        R[i+1] = R[i] + dt*(delta*S[i] + delta*I[i] + alpha*S[i]*Z[i] - zeta*R[i]); 
    }
}
"""

N = 100
time_loop = inline_with_numpy(c_code, arrays = [['nS', 'S'], ['nI', 'I'], ['nZ', 'Z'], ['nR', 'R']], cache_dir = "zombie_cache")
S = zeros(N)
I = zeros(N)
Z = zeros(N)
R = zeros(N)

T = 100 
dt = T/(N-1)
PI = 0.0 
alpha = 0.05 
beta = 0.095
zeta = 0.01 
delta = 0.01 
rho = 0.001 

S0 = 50
Z0 = 1 
R0 = 0 
I0 = 0 

time_loop(S, I, Z, R, PI, alpha, beta, delta, rho, zeta, dt, N, S0, I0, Z0, R0)


tt = arange(0, 1, 1.0/N) 

try: 
    import pylab
    pylab.plot(tt, S)
#    pylab.plot(tt, I)
    pylab.plot(tt, Z)
#    pylab.plot(tt, R)
    #pylab.savefig('pressure_plot')
    pylab.show()
except: 
    print("To get a plot of the solution install pylab")







"""This module contains helper functions for working with the module cache.

Example operations:
  - modulename = modulename_from_checksum(checksum)
  - modulename = modulename_from_checksum(compute_checksum(signature))
  - module = import_module_directly(path, modulename)
  - module = import_module(modulename)
  - module = import_module(checksum)
  - module = import_module(compute_checksum(signature))
  - modules = cached_modules()
  - modules = cached_modules(cache_dir)
"""

# Copyright (C) 2008 Martin Sandve Alnes
#
# This file is part of Instant.
#
# Instant is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Instant is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Instant. If not, see <http://www.gnu.org/licenses/>.
#
# Alternatively, Instant may be distributed under the terms of the BSD license.

import os, sys, re
from .output import instant_warning, instant_assert, instant_debug
from .paths import get_default_cache_dir, validate_cache_dir
from .signatures import compute_checksum

# TODO: We could make this an argument, but it's used indirectly
# several places so take care.
_modulename_prefix = "instant_module_"
def modulename_from_checksum(checksum):
    "Construct a module name from a checksum for use in cache."
    return _modulename_prefix + checksum


def checksum_from_modulename(modulename):
    "Construct a module name from a checksum for use in cache."
    return modulename.remove(_modulename_prefix)


def import_module_directly(path, modulename):
    "Import a module with the given module name that resides in the given path."
    sys.path.insert(0, path)
    er = None
    try:
            module = __import__(modulename)
    except BaseException as e:
        instant_warning("In instant.import_module_directly: Failed to import module '%s' from '%s';\n%s:%s;" % (modulename, path, type(e).__name__, e))
        module = None
        er = e
    finally:
        sys.path.pop(0)
    return module, er


_memory_cache = {}
def memory_cached_module(moduleid):
    "Returns the cached module if found."
    import sys
    module = _memory_cache.get(moduleid, None)
    instant_debug("Found '%s' in memory cache with key '%r'." % (module,
                                                                 moduleid))
    return module


def place_module_in_memory_cache(moduleid, module):
    "Place a compiled module in cache with given id."
    _memory_cache[moduleid] = module
    instant_debug("Added module '%s' to cache with key '%r'." % (module,
                                                                 moduleid))


def is_valid_module_name(name):
    NAMELENGTHLIMIT = 200
    return len(name) < NAMELENGTHLIMIT and bool(re.search(r"^[a-zA-Z_][\w]*$",
                                                          name))


def import_and_cache_module(path, modulename, moduleids):
    module, e = import_module_directly(path, modulename)
    instant_assert(module is not None, "Failed to import module found in cache. Modulename: '%s';\nPath: '%s';\n%s:%s;" % (modulename, path, type(e).__name__,
                                                                                                                           e))
    for moduleid in moduleids:
        place_module_in_memory_cache(moduleid, module)
    return module


def check_memory_cache(moduleid):
    # Check memory cache first with the given moduleid
    moduleids = [moduleid]
    module = memory_cached_module(moduleid)
    if module: return module, moduleids

    # Get signature from moduleid if it isn't a string,
    # and check memory cache again
    if hasattr(moduleid, "signature"):
        moduleid = moduleid.signature()
        instant_debug("In instant.check_memory_cache: Got signature "\
                      "'%s' from moduleid.signature()." % moduleid)
        module = memory_cached_module(moduleid)
        if module:
            # FIXME (GNW): I haved commented this out since it can continually
            #              insert indentical objects into the cache. This
            #              function checks the cache, so why should it also
            #              insert?
            #for moduleid in moduleids:
            #    place_module_in_memory_cache(moduleid, module)
            return module, moduleids
        moduleids.append(moduleid)

    # Construct a filename from the checksum of moduleid if it
    # isn't already a valid name, and check memory cache again
    if not is_valid_module_name(moduleid):
        moduleid = modulename_from_checksum(compute_checksum(moduleid))
        instant_debug("In instant.check_memory_cache: Constructed module name "\
                      "'%s' from moduleid '%s'." % (moduleid, moduleids[-1]))
        module = memory_cached_module(moduleid)
        if module: return module, moduleids
        moduleids.append(moduleid)

    instant_debug("In instant.check_memory_cache: Failed to find module: %s." % moduleid)
    return None, moduleids


def check_disk_cache(modulename, cache_dir, moduleids):
    # Ensure a valid cache_dir
    cache_dir = validate_cache_dir(cache_dir)

    # Check on disk, in current directory and cache directory
    for path in (os.getcwd(), cache_dir):
        if os.path.exists(os.path.join(path, modulename, "finished_copying")):

            # Found existing directory, try to import and place in memory cache
            module = import_and_cache_module(path, modulename, moduleids)
            if module:
                instant_debug("In instant.check_disk_cache: Imported module "\
                              "'%s' from '%s'." % (modulename, path))
                return module
            else:
                instant_debug("In instant.check_disk_cache: Failed to import "\
                              "module '%s' from '%s'." % (modulename, path))

    # All attempts failed
    instant_debug("In instant.check_disk_cache: Can't import module with modulename "\
                  "%r using cache directory %r." % (modulename, cache_dir))
    return None


def import_module(moduleid, cache_dir=None):
    """Import module from cache given its moduleid and an optional cache directory.

    The moduleid can be either
      - the module name
      - a signature string, of which a checksum is taken to look up in the cache
      - a checksum string, which is used directly to look up in the cache
      - a hashable non-string object with a function moduleid.signature() which is used to get a signature string
    The hashable object is used to look up in the memory cache before signature() is called.
    If the module is found on disk, it is placed in the memory cache.
    """
    # Look for module in memory cache
    module, moduleids = check_memory_cache(moduleid)
    if module: return module

    # Look for module in disk cache
    modulename = moduleids[-1]
    return check_disk_cache(modulename, cache_dir, moduleids)


def cached_modules(cache_dir=None):
    "Return a list with the names of all cached modules."
    cache_dir = validate_cache_dir(cache_dir)
    return os.listdir(cache_dir)



from __future__ import print_function
import pytest
import numpy as N
import time
from instant import inline_with_numpy


def test_grid_loop():

    func_str = "sin"
    c_code = """
void gridloop(int x1, int y1, double* a,
              int n, double* x,
              int m, double* y) {
  for (int i=0; i<n; i++) {
      for (int j=0; j<m; j++) {
          a[i*n +j] = %s(x[i] + y[j]);
      }
  }
}
    """ % func_str

    n = 5000

    a = N.zeros([n, n])
    x = N.arange(0.0, n, 1.0)
    y = N.arange(0.0, n, 1.0)

    arrays = [['x1', 'y1', 'a'], ['n', 'x'], ['m', 'y']]
    grid_func = inline_with_numpy(c_code, arrays=arrays )


    t1 = time.time()
    grid_func(a, x, y)
    t2 = time.time()
    print('With instant:', t2-t1, 'seconds')


    xv = x[:, N.newaxis]
    yv = y[N.newaxis,:]
    a2 = N.zeros([n, n])
    t1 = time.time()
    a2[:,:] = N.sin(xv + yv)
    t2 = time.time()
    print('With numpy:', t2-t1, 'seconds')

    d = a-a2
    d.shape = (n*n,)

    assert abs(max(d)) < 1.0e-12



from __future__ import print_function
import pytest
import numpy
import time
from instant import inline_with_numpy

def test_inline_numpy1():

    # Example 1: two arrays, one in, one inout
    c_code = """
double sum (int x1, int y1, int z1, double* array1, int x2, double* array2){
  double tmp = 0.0;
  for (int i=0; i<x1; i++)
    for (int j=0; j<y1; j++)
      for (int k=0; k<z1; k++){
        tmp += array1[i*y1*z1 + j*z1 + k];
        array2[1] = 2;
    }
  return tmp;
}
    """

    sum_func = inline_with_numpy(c_code, arrays = [['x1', 'y1', 'z1', 'array1'],
                                                   ['x2', 'array2', 'in']],
                                 cache_dir="test_ex1_cache")

    a = numpy.arange(27000); a = numpy.sin(a)
    c = a.copy()
    a.shape = (30, 30, 30)

    #print('b = (1,1)')
    b = (1., 1.)
    sum1 = sum_func(a, b)
    #print(b, 'b not changed when list')

    b = numpy.ones(2)
    sum1 = sum_func(a, b)
    #print(b, 'b not changed when numpy array')
    #print(sum1)


def test_inline_numpy2():

    # Example 2: two array, both inout and of same size
    # Cannot avoid specifying all dimensions for both arrays
    c_code = """
double sum (int x1, int y1, double* array1, int x2, int y2, double* array2){
  double tmp = 0.0;
  for (int i=0; i<x1; i++)
    for (int j=0; j<y1; j++){
      tmp = array1[i*y1 + j];
      array1[i*y1 + j] = array2[i*y1 + j];
      array2[i*y1 + j] = tmp;
    }
  return tmp;
}
    """

    sum_func = inline_with_numpy(c_code, arrays = [['x1', 'y1', 'array1'],
                                                   ['x2', 'y2', 'array2']],
                                 cache_dir="test_ex2_cache")

    a = numpy.ones(4)
    a.shape = (2, 2)
    b = a.copy()
    a *= 2

    sum1 = sum_func(a, b)
    #print('a and b changed')
    #print(a)
    #print(b)


def test_inline_numpy3():
    # Example 3: two arrays, one in, one out
    c_code = """
void sum (int x1, int y1, double* array1, int xy2, double* array2){
  for (int i=0; i<x1; i++)
    for (int j=0; j<y1; j++)
      array2[i*y1 + j] = array1[i*y1 + j]*2;
}
    """

    sum_func = inline_with_numpy(c_code, arrays = [['x1', 'y1', 'array1', 'in'],
                                                   ['xy2', 'array2', 'out']],
                                 cache_dir="test_ex3_cache")

    a = numpy.ones(4)
    a.shape = (2, 2)
    a *= 2

    c = sum_func(a, a.size)
    c.shape = a.shape
    #print(c.shape)


def test_inline_numpy4():
    # Example 4: three arrays, one in, one inout, and one out
    c_code = """
void sum (int x1, int y1, long* array1, int x2, int* array2, int x3, double* array3){
  for (int i=0; i<x1; i++){
    array3[i] = 0;
    for (int j=0; j<y1; j++)
      array3[i] += array1[i*y1 + j]*array2[j];
  }
}
    """

    sum_func = inline_with_numpy(c_code, arrays = [['x1', 'y1', 'array1', 'in', 'long'],
                                               ['x2', 'array2', 'int'],
                                               ['x3', 'array3', 'out', 'double']],
                                 cache_dir="test_ex4_cache")

    a = numpy.arange(9)#, dtype='int32')
    a.shape = (3, 3)
    b = numpy.arange(3, dtype='int32')

    c = sum_func(a, b, b.size)
    #print(c)
    #print(numpy.dot(a, b))


def test_inline_numpy5():
    # Example 5: arrays with more than 3 dimensions, uses old typemaps, only doubles
    c_code = """
void sum (int m, int* mp, double* array1, int n, int* np, double* array2){
  int w = mp[0], x = mp[1], y = mp[2], z = mp[3];
  for (int i=0; i<w; i++)
    for (int j=0; j<x; j++)
      for (int k=0; k<y; k++)
        for (int l=0; l<z; l++){
          *array2 = *array1*2;
          array1++;
          array2++;
        }
}
    """

    sum_func = inline_with_numpy(c_code, arrays = [['m', 'mp', 'array1', 'multi'],
                                                   ['n', 'np', 'array2', 'multi'],],
                                 cache_dir="test_ex5_cache")

    a = numpy.arange(16, dtype='float64')
    a.shape = (2, 2, 2, 2)
    b = a.copy()*0

    sum_func(a, b)
    assert numpy.array(a*2 == b).all()



from __future__ import print_function
import os
from instant import inline

def test_system_call_subprocess():
    os.environ["INSTANT_SYSTEM_CALL_METHOD"] = "SUBPROCESS"
    add_func = inline("double add(double a, double b){ return a+b; }", cache_dir="test_cache")
    print("The sum of 3 and 4.5 is ", add_func(3, 4.5))


def test_system_call_os_system():
    os.environ["INSTANT_SYSTEM_CALL_METHOD"] = "OS_SYSTEM"
    add_func = inline("double add(double c, double d){ return c+d; }", cache_dir="test_cache")
    print("The sum of 3 and 4.5 is ", add_func(3, 4.5))



from __future__ import print_function
from instant import inline_with_numpy
from numpy import *
import time


def time_loop2(p, Q, A, B, dt, N, p0):
    p[0] = p0
    for i in range(1, N):
        p[i] = p[i-1] + dt*(B*Q[i] - A*p[i-1])

c_code = """
void time_loop(int n, double* p,
             int m, double* Q,
             double A, double B,
             double dt, int N, double p0)
{
    if ( n != m ) {
        printf("n and m should be equal");
        return;
    }
    if ( n != N ) {
        printf("n and N should be equal");
        return;
    }

    p[0] = p0;
    for (int i=1; i<n; i++) {
        p[i] = p[i-1] + dt*(B*Q[i] - A*p[i-1]);
    }
}
"""

N = 100000
time_loop = inline_with_numpy(c_code, arrays = [['n', 'p'], ['m', 'Q']], cppargs='-g', cache_dir="test_cache")
p = zeros(N)
T = 20.0
Q = sin(arange(0, T, T/N))+1

t1 = time.time()
time_loop(p, Q, 1.0, 1.0, 1.0/N, N, 1.0)
t2 = time.time()
print('With instant:', t2-t1, 'seconds')

p2 = zeros(N)
t1 = time.time()
time_loop2(p2, Q, 1.0, 1.0, 1.0/N, N, 1.0)
t2 = time.time()
print('With Python :', t2-t1, 'seconds')

print('The max difference between p and p2 is ', max(abs(p - p2)))


tt = arange(0, 1, 1.0/N)

a = """
try:
    import pylab
    pylab.plot(tt, p)
    pylab.plot(tt, Q)
    pylab.xlabel('time ')
    pylab.ylabel('pressure ')
    pylab.title('Pressure ')
    #pylab.savefig('pressure_plot')
    pylab.show()
except:
    print "To get a plot of the solution install pylab"
"""



"""
Instant allows compiled C/C++ modules to be created
at runtime in your Python application, using SWIG to wrap the
C/C++ code.

A simple example:
    >>> from instant import inline
    >>> add_func = inline(\"double add(double a, double b){ return a+b; }\")
    >>> print "The sum of 3 and 4.5 is ", add_func(3, 4.5)

The main functions are C{build_module}, C{write_code}, and
C{inline*} see their documentation for more details.

For more examples, see the tests/ directory in the Instant distribution.

Questions, bugs and patches should be sent to fenics-dev@googlegroups.com.
"""

import pkg_resources

__authors__ = "Magne Westlie, Kent-Andre Mardal <kent-and@simula.no>, Martin Alnes <martinal@simula.no>, Ilmar M. Wilbers <ilmarw@simula.no>"
__date__ = "2016-11-30"
__version__ = pkg_resources.get_distribution("instant").version

# TODO: Import only the official interface
from .output import *
from .config import *
from .paths import *
from .signatures import *
from .cache import *
from .codegeneration import *
from .build import *
from .inlining import *
